#! /usr/bin/env python
"""
Model Management and simulation objects.
"""
import cPickle, time
import os, codecs,  subprocess
from copy import deepcopy
from xmlrpclib import ServerProxy, Error
from simobj import graph, edge, siteobj
from numpy import *
import spread
from numpy.random import uniform, poisson
from sqlobject import *
from math import *
from data_io import *
from optparse import OptionParser
import ConfigParser
import string, sys, getpass
import report as Rp
import epigdal
import __version__
import encodings.utf_8
import encodings.latin_1
import dataObject as DO
import sqlite3, MySQLdb
#import pycallgraph

# Import Psyco if available
#try:
#    import psyco
#
#    psyco.full()
#except ImportError:
#    pass

class simulate:
    """
    This class takes care of setting up the model, simulating it, and storing the results
    """
    def __init__(self, fname=None, host='localhost',port=3306,db='epigrass',user='epigrass', password='epigrass',backend = 'mysql'):
        #pycallgraph.start_trace()
        self.host = host
        self.port = port
        self.usr = user
        self.db = db
        self.fname = fname
        self.backend = backend
        self.passw = password
        self.repname = None
        self.shapefile = None
        self.World = None
        self.shpout = True
        self.dir = os.getcwd()
        sys.path.insert(0,self.dir) #add current path to sys.path
        self.gui = 0 # True if this object was created by the gui
        self.now = time.asctime().replace(' ','_').replace(':','') #current date and time
        self.modelName = os.path.split(self.fname)[-1].split('.')[0]
        self.config = self.loadModelScript(self.fname)
        self.evalConfig(self.config)
        if not self.outdir:
            self.outdir = outdir = 'outdata-'+self.modelName
        if not os.path.exists(self.outdir):
            os.mkdir(self.outdir)
        if self.shapefile: #create world object if a shapefile is provided
            self.World = epigdal.World(*self.shapefile+[self.outdir])
        self.chkScript()
        self.encoding = 'latin-1'#default encoding for strings in the sites and edges files
        self.round = 0 #No replicates
        sitios = loadData(self.sites, sep=',', encoding=self.encoding)
        ed = loadData(self.edges,sep=',', encoding=self.encoding)
        l = self.instSites(sitios)
        e = self.instEdges(l,ed)
        self.g = self.instGraph(self.modelName,1,l,e)
        #if self.layer:
        #    pass
            #self.grassVect2ascii(self.layer)

        #self.g.plotDegreeDist()


    def loadModelScript(self,fname):
        """
        Loads the model specification from the text file.
        Returns a dictionary with keys of the form
        <section>.<option> and the corresponding values.
        """
        config = {}
        config = config.copy()
        cp =ConfigParser.SafeConfigParser()
        cp.read(fname)
        for sec in cp.sections():
            name =  string.lower(sec)
            for opt in cp.options(sec):
                config[name+"."+string.lower(opt)] = string.strip(cp.get(sec,opt))
        return config

    def evalConfig(self,config):
        """
        Takes in the config dictionary and generates the global variables needed.
        """
        try:
            #WORLD
            #load Shapefile if specified
            if config['the world.shapefile']:
                self.shapefile = eval(config['the world.shapefile'])
            else:
                self.shapefile = []

            #self.layer = config['the world.vector layer']
            self.sites = config['the world.sites'] #file containing site info
            self.edges = config['the world.edges'] #file containing edge info
            if config['the world.encoding']:
                self.encoding = config['the world.encoding']#encoding specification
            #Epidemiological model
            self.modtype = config['epidemiological model.modtype']
            #self.models = eval(config['epidemiological model.models'])
            #PARAMETERS
            self.beta = config['model parameters.beta']
            self.alpha = config['model parameters.alpha']
            self.e = config['model parameters.e']
            self.r = config['model parameters.r']
            self.delta = config['model parameters.delta']
            self.B = config['model parameters.b']
            self.w = config['model parameters.w']
            self.p = config['model parameters.p']
            #INITIAL CONDITIONS  (inits are string to evaluated inside siteobj.createmodel)
            self.S = config['initial conditions.s']
            self.E = config['initial conditions.e']
            self.I = config['initial conditions.i']
            #EPIDEMIC EVENTS
            self.seed = eval(config['epidemic events.seed'])
            self.vaccinate = eval(config['epidemic events.vaccinate'])
            #self.quarantine = eval(config['epidemic events.quarantine'])

            #[TRANSPORTATION MODEL]
            self.doTransp = eval(config['transportation model.dotransp'])
            self.stochTransp = eval(config['transportation model.stochastic'])
            self.speed = eval(config['transportation model.speed'])

            #SIMULATION AND OUTPUT
            self.steps = eval(config['simulation and output.steps'])
            self.outdir = config['simulation and output.outdir']
            self.MySQLout = eval(config['simulation and output.mysqlout'])
#            self.Rep = eval(config['simulation and output.report'])
#            self.siteRep = eval(config['simulation and output.siterep'])
            self.replicas = eval(config['simulation and output.replicas'])
            self.randomize_seeds = eval(config['simulation and output.randseed'])
            self.Batch = eval(config['simulation and output.batch'])
        except KeyError, v:
            V = v.__str__().split('.')
            sys.exit("Please check the syntax of your '.epg' file.\nVariable %s, from section %s was not specified."%(V[1],V[0]))
        if self.replicas:
#            self.Rep = 0 #Turns off reports
            self.Batch = []#Turns off Batch mode
            self.round = 0# Initialize replicate counter
    # generate dictionaries for parameters and inits
        self.inits = {}
        self.parms = {}
        for k,v in config.items():
            if k.startswith('initial conditions'):
                self.inits[k.split('.')[-1]] = v
            elif k.startswith('model parameters'):
                self.parms[k.split('.')[-1]] = v
            else:pass

    def chkScript(self):
        """
        Checks the type of the variables on the script
        """
        self.Say("Checking syntax of model script... NOW")

        if not os.access(self.sites,os.F_OK):
            self.Say('Sites file %s does not exist, please check your script.'%self.sites)
            sys.exit()
        if not os.access(self.edges,os.F_OK):
            self.Say('Egdes file %s does not exist, please check your script.'%self.edges)
            sys.exit()
        if not self.modtype in ['SIS','SIS_s','SIR','SIR_s','SEIS','SEIS_s','SEIR','SEIR_s','SIpRpS','SIpRpS_s','SIpR,SIpR_s','Influenza','Custom']:
            self.Say('Model type %s is invalid, please check your script.'%self.modtype)




        self.Say('Script %s passed syntax check NOW.'%self.modelName)
    def deg2dec(self,coord):
        """
        converts lat/long to decimal
        """
        co = coord.split(':')
        if int(co[0])<0:
            result = float(co[0])-float(co[1])/60-float(co[2])/3600
        else:
            result = float(co[0])+float(co[1])/60+float(co[2])/3600

        return result


    def instSites(self,sitelist):
        """
        Instantiates and returns a list of siteobj instances, from a list of site specification
        as returned by data_io.loadData.
        Here the site specific events are passed to each site, and the site models are created.
        """
        header = sitelist.pop(0)
        objlist=[]
        for site in sitelist:
            if ':' in site[0]:
                lat = self.deg2dec(site[0])
                long = self.deg2dec(site[1])
            else:
                lat = float(site[0])
                long = float(site[1])
            objlist.append(siteobj(site[2],site[3],(lat,long),int(strip(site[4])),tuple([float(strip(i)) for i in site[5:]])))
        for o in objlist:
            if self.stochTransp: o.stochtransp = 1
            N = o.totpop #local copy for reference on model creation
            values = o.values #local copy for reference on model creation
            inits={}
            parms={}
            #eval parms and inits
            for k,v in self.inits.items():
                inits[k] = eval(v)
            for k,v in self.parms.items():
                parms[k] = eval(v)
            #Linhas abaixo estao obsoletas (remover quando tiver tempo para testar)
            I = eval(self.I)
            E = eval(self.E)
            S = eval(self.S)
            beta = eval(self.beta)
            alpha = eval(self.alpha)
            e = eval(self.e)
            r = eval(self.r)
            delta = eval(self.delta)
            B = eval(self.B)
            w = eval(self.w)
            p = eval(self.p)
            #################
            if self.vaccinate:
                if self.vaccinate[0][0] == 'all':
                    o.vaccination = [self.vaccinate[0][1],self.vaccinate[0][2]]
                else:
                    for i in self.vaccinate:
                        if int(o.geocode) == i[0]:
                            o.vaccination = [i[1],float(i[2])]
#            if self.quarantine:
#                for i in self.quarantine:
#                    if int(o.geocode) == i[0]:
#                        o.quarantine == i[1:]

            if self.seed:
                #print type(o.geocode), type(self.seed[0])
                if self.seed[0][0] == "all":
                    I += j[2]
                    inits[self.seed[0][1].lower()] += 1
                for j in self.seed:
                    if int(o.geocode) == j[0]:
                        #self.Say("%s infected person(s) arrived at %s"%(j[2],o.sitename))
                        I += j[2]
                        inits[j[1].lower()] +=j[2]
                        o.createModel((E,I,S),self.modtype,self.modelName,v=values,bi=inits,bp=parms)
                    else:
                        o.createModel((E,I,S),self.modtype,self.modelName,v=values,bi=inits,bp=parms)
            else:
                o.createModel((E,I,S),self.modtype,self.modelName,v=values,bi=inits,bp=parms)

        return objlist

    def instEdges(self,sitelist, edgelist):
        """
        Instantiates and returns a list of edge objects,

        sitelist -- list of siteobj objects.

        edgelist -- list of edge specifications as returned by data_io.loadData.
        """
        header = edgelist.pop(0)
        #print [i.sitename for i in sitelist]
        objlist=[]
        source = dest = None
        for edg in edgelist:
            #print edgelist.index(edg)
            if edg[5] == edg[6]:
                #ignore circular edges
                continue
            for site in sitelist:
                #print site.sitename
                if str(site.geocode) == edg[5]:
                    source = site
                    #print 'found source = ',source.sitename
                elif str(site.geocode) == edg[6]:
                    dest = site
                    #print 'found dest = ',dest.sitename
                if (source and dest):
                    #print 'next edge!'
                    break
            if not (source and dest):
                #print source ,dest
                sys.exit('One of the vertices on edge '+edg[0]+'-'+edg[1]+' could not be found on the site list')

            objlist.append(edge(source,dest,edg[2],edg[3],float(edg[4])))
            source = dest = None


        return objlist

    def instGraph(self,name, digraph, siteobjlist, edgeobjlist):
        """
        Instantiates and returns a graph object from a list of edge objects.
        """
        g = graph(name,digraph)
        g.speed = self.speed

        for j in siteobjlist:
            g.addSite(j)

        for i in edgeobjlist:
            g.addEdge(i)

        return g

    def randomizeSeed(self, option):
        """
        Generate and return a list of randomized seed sites
        to be used in a repeated runs scenario.
        Seed sites are sampled with a probability proportional to the
        log of their population size.
        option: if 1 randomize; if 2, return unrundomized sequence
        """
        if option==1:
            poplist = [log10(site.totpop) for site in self.g.site_list]
            lpl = len(poplist)
            popprob = array(poplist)/sum(poplist) #acceptance probability
            u = floor(uniform(0,lpl, self.replicas))
            sites=[]
            i=0
            while i < self.replicas:
                p = uniform(0,1)
                if p <= popprob[int(u[i])]:
                    sites.append(self.g.site_list[int(u[i])])
                    i += 1
        elif option ==2:
            sites = self.g.site_list
        return sites

    def setSeed(self,seed,n=1):
        '''
        Resets the number of infected to zero in all nodes but seed,
        which get n infected cases.
        seed must be a siteobj object.
        '''
        #inits[self.seed[0][1].lower()] += 1
        seedvar = self.seed[0][1].lower() #retrieve the name of the variable containing the seeds
        self.Say('seedvar= %s'%seedvar)

        self.seed =[(seed.geocode, seedvar,n)]
        for site in self.g.site_list:
            if site.geocode == seed.geocode:
                site.bi[seedvar] = n
#                site.ts[0][seedpos] = n

                self.Say("%s infected case(s) arrived at %s"%(n,seed.sitename))
            else:
                site.bi[seedvar] = 0
#                site.ts[0][seedpos] = 0

    def start(self):
        """
        Start the simulation
        """
        if '/' in self.modelName:
            self.modelName = os.path.split(self.modelName)[-1]
#        print "====>", self.gui
#        if self.gui and not self.round: #open the display window only on the first round of a reprun
#            subprocess.Popen(['python','Epigrass/dgraph.py'])
#            self.dserver = ServerProxy("http://localhost:50000", allow_none = True) # local display server
        time.sleep(10)
        self.Say('Simulation starting.')
        start = time.time()
        self.runGraph(self.g,self.steps,transp=self.doTransp)
        elapsed = time.time()-start
        self.Say('Simulation lasted %s seconds.'%elapsed)
        if self.MySQLout:
            if self.backend == 'csv':
                self.outToCsv(self.modelName)
            else:
                self.outToDb(self.modelName)
        if self.shpout:
            self.outToShp()
        self.dumpData()
        spread.Spread(self.g,  self.outdir, self.encoding)
        #self.saveModel(self.modelName)
        #pycallgraph.make_dot_graph(self.modelName+'_callgraph.png')
        #print len(self.g.site_list[-1].ts)

##        for i in self.g.site_list:
##            i.plotItself()
##        show()

    def createDataFrame(self,site):
        """
        Saves the time series *site.ts* to outfile as specified on the
        model script.
        """
        data = array(site.ts, Float)
        inc  = site.incidence
        f = open(self.outdir+self.outfile,'w')
        f.write('time,E,I,S,incidence\n')
        try:
            for t,i in enumerate(data):
                j = [str(k) for k in i]
                line = str(t)+','.join(j) + str(inc[t])+'\n'
                f.write(line)
        finally:
            f.close()

        return 'E,I,S\n'+tss

    def grassVect2ascii(self,layer):
        """
        Convert Grass vector layer to an ascii file epigrass can read
        Deprecated!
        """
        com = 'v.out.ascii input=%s output=map.txt format=standard'%layer
        os.system(com)

    def outToShp(self):
        """
        Creates Data Layers in shapefile format
        """
        if not self.World:
            return
        #Initialize world output layers
        self.Say("REading Nodes from shapefile...")
        self.World.getNodeList(self.World.ds.GetLayerByName(self.World.layerlist[0]))
        self.Say("Done reading nodes!")
        self.Say("Creating Nodes shapefile...")
        self.World.createNodeLayer()
        self.Say("Done creating Nodes shapefile!")
        elist = [(e.source.geocode,e.dest.geocode,sum(e.ftheta),sum(e.btheta)) for e in self.g.edge_list]
        self.Say("Creating Edges shapefile...")
        self.World.createEdgeLayer(elist)
        self.Say("Done creating Edges shapefile!")
        #Generate site epidemic stats
        varlist = ["prevalence","totalcases", "arrivals","population"]
        sitestats = []
        names = {}
        for site in self.g.site_list:
            names[int(site.geocode)] = site.sitename
            prevalence = float(site.totalcases)/site.totpop
            #print prevalence
            sitestats.append((site.geocode,prevalence,site.totalcases,sum(site.thetahist),float(site.totpop)))

        self.Say("Creating Data shapefile...")
        self.World.createDataLayer(varlist,sitestats)
        self.Say("Done creating Data shapefile!")
        #Generate the kml too.
        self.Say("Creating KML output file...")
        lr = self.World.datasource.GetLayer(0)
        k = epigdal.KmlGenerator()
        k.addNodes(lr,names)
        k.writeToFile(self.outdir)
        self.Say("Done creating KML file!")
        #close files
        self.World.closeSources()



    def writeMetaCSV(self, table):
        """
        Writes a meta CSV table
        """
        if not self.outdir == os.getcwd():
            os.chdir(self.outdir)
        f = open(table+'_meta', 'w')
        cfgitems = self.config.items()
        h = ';'.join([k.strip().replace(' ','_').replace('.', '$') for k,v in cfgitems])
        l = ';'.join([v.split('#')[0] for k, v in cfgitems])
        f.write(h+'\n')
        f.write(l+'\n')
        f.close()
        os.chdir(self.dir)

    def outToCsv(self,table):
        """
        Save simulation results in csv file.
        """
        if not self.outdir == os.getcwd():
            os.chdir(self.outdir)
        tablee = table+"_"+self.now+"_e.tab" #edgetable name
        table += "_"+self.now+".tab"#sitetable name
        f = open(table,"w")
        head = ['geocode','time','totpop','name',
        'lat','longit']
        headerwritten = False
        for site in self.g.site_list:
            t = 0
            regb = [str(site.geocode),str(t),str(site.totpop),
                site.sitename.replace('"','').encode('ascii','replace'),
                str(site.pos[0]),str(site.pos[1])
                ]
            if site.values:
                for n,v in enumerate(site.values):
                    head.append('values%s'%n)
                    regb.append(str(v))
            #print site.sitename, site.ts
            ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
            head.extend(['incidence','arrivals'])
            for n,v in enumerate(site.vnames):
                    head.append(str(v))
            for i in ts:
                reg = deepcopy(regb)
                reg.extend([str(site.incidence[t]),str(site.thetahist[t])])
                reg[1] = str(t)
                for n,v in enumerate(site.vnames):
                    reg.append(str(i[n]))
                if not headerwritten:
                    h =",".join(head)
                    f.write(h+'\n')
                    headerwritten = True
                f.write(','.join(reg)+'\n')
                t += 1
        f.close()
        #inserting data in edges file
        g = open(tablee,'w')
        t=0
        head = ['source_code','dest_code','time','ftheta','btheta']
        ehw = False
        for e in self.g.edge_list:
            for f,b in zip(e.ftheta,e.btheta):
                if not ehw:
                    g.write(','.join(head)+'\n')
                    ehw = True
                ereg = [e.source.geocode,e.dest.geocode,t,f,b]
                ereg = [str(i) for i in ereg]
                g.write(','.join(ereg)+'\n')
                t += 1
        g.close()
        os.chdir(self.dir)

    def outToODb(self,table,mode='b'):
        """
        Insert simulation results in a database using sqlobject.
        if mode = b, do batch inserts
        if mode = p, do parallel inserts
        """
        #set connection parameters
        try:
            DO.Connect(self.backend.lower(),self.usr,self.passw,self.host,self.port,self.db)
            self.Say('Saving data on %s database...'%self.backend)
        except:
            self.Say('Database Connection Failed')
        #basic connection variables
        name = table+'_'+self.now
        self.outtable = name

        DO.Site.sqlmeta.table = name
        DO.Edge.sqlmeta.table = name+'e'
        if self.backend.lower()=='sqlite':
            DO.Site.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
            DO.Site._connection.commit()
            DO.Edge.sqlmeta.createSQL = {'sqlite':['PRAGMA synchronous=OFF;','PRAGMA default_cache_size=12000000;']}
            DO.Edge._connection.commit()
        #adding columns to sites table
        try: #Only add if they don't exist. On Batch runs this will catch the add error
            DO.Site.sqlmeta.addColumn(StringCol('incidence'))
            DO.Site.sqlmeta.addColumn(StringCol('arrivals'))
        except: pass
        if self.g.site_list[0].values:#add columns for 'values' variables
            for i in xrange(len(self.g.site_list[0].values)):
                try:
                    DO.Site.sqlmeta.addColumn(StringCol('values%s'%i))
                except: pass
        for s in self.g.site_list[0].vnames: #add columns for state variables
            try:
                DO.Site.sqlmeta.addColumn(StringCol(s))
            except: pass
        #creating tables
        DO.Site.createTable()
        DO.Edge.createTable()
        #saving pickle of adjacency matrix
        matname = 'adj_'+self.modelName # table
        adjfile = open(matname,'w')
        cPickle.dump(self.g.getConnMatrix(),adjfile)
        adjfile.close()
        #calling insert function
        if mode == 'b':
            self.batchInsert()
        elif mode == 'p':
            self.parInsert()

    def batchInsert(self):
        """
        Do the inserts all at once. After the simulation is done
        """
        #inserting data in sites table
        for site in self.g.site_list:
            dicin={
            'geocode':site.geocode,
            'lat':float(site.pos[0]),
            'longit':float(site.pos[1]),
            'name':site.sitename.replace('"','').encode('ascii','replace'),
            'totpop':int(site.totpop)}
            if site.values:
                for n,v in enumerate(site.values):
                    #print v,type(v)
                    dicin['values%s'%n] = str(v)
            ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
            t = 0
            for i in ts:
                dicin['incidence']= str(site.incidence[t])
                dicin['arrivals'] = str(site.thetahist[t])
                dicin['time'] = t
                for n,v in enumerate(site.vnames):
                    #print i[n],type( i[n])
                    dicin[v] = str(i[n])
                    DO.Site(**dicin)
                t += 1
        #DO.Site._connection.commit()
        #inserting data in edges table
        for e in self.g.edge_list:
            rlist = []
            edicin={
            'source_code':e.source.geocode,
            'dest_code':e.dest.geocode}
            t=0
            for f,b in zip(e.ftheta,e.btheta):
                edicin['time'] = t
                edicin['ftheta'] = f
                edicin['btheta'] = b
                rlist.append(edicin)
                t += 1
            insobj = DO.sqlbuilder.Insert(DO.Edge.sqlmeta.table,valueList=rlist)
            DO.Edge._connection.queryAll(DO.Edge._connection.sqlrepr(insobj))
            #DO.Edge._connection.commit()

    def parInsert(self):
        """
        do the insertions in a separate process.
        """
        pid = os.fork()
        if pid:
            #parent process get on with the simulation
            return
        else:
        #child process inserts data in sites table
            time.sleep(1+self.g.simstep)
            for site in self.g.site_list:
                dicin={
                'geocode':site.geocode,
                'lat':float(site.pos[0]),
                'longit':float(site.pos[1]),
                'name':site.sitename.replace('"','').encode('ascii','replace'),
                'totpop':int(site.totpop)}
                if site.values:
                    for n,v in enumerate(site.values):
                        #print v,type(v)
                        dicin['values%s'%n] = str(v)
                ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
                t = self.g.simstep
                dicin['incidence']= str(site.incidence[t])
                dicin['arrivals'] = str(site.thetahist[t])
                dicin['time'] = t
                for n,v in enumerate(site.vnames):
                    #print i[n],type( i[n])
                    dicin[v] = str(i[n])
                DO.Site(**dicin)
            DO.Site._connection.commit()
            #inserting data in edges table
            for e in self.g.edge_list:
                edicin={
                'source_code':e.source.geocode,
                'dest_code':e.dest.geocode}
                edicin['time'] = t
                edicin['ftheta'] = e.ftheta[t]
                edicin['btheta'] = e.btheta[t]
                DO.Edge(**edicin)
            DO.Edge._connection.commit()
            sys.exit("commit successful")

    def writeMetaTable(self, table):
        """
        Creates a Meta-Info Table on the database with the contents of the .epg file
        """
        try:
            table = table+'_meta'
            if self.backend.lower() == "mysql":
                con = MySQLdb.connect(host=self.host, port=self.port,
                                    user=self.usr,passwd=self.passw, db=self.db)
            elif self.backend.lower() == "sqlite":
                if not self.outdir == os.getcwd():
                    os.chdir(self.outdir)
                con = sqlite3.connect("Epigrass.sqlite")
                os.chdir(self.dir)
            # Create table
            sqlstr1 = "CREATE TABLE %s("%table
            vars = []
            cfgitems = self.config.items()
            for k, v in cfgitems:
                vars.append(k.strip().replace(' ','_').replace('.', '$'))
            sqlstr2 = ', '.join(["%s text"%i for i in vars])
            Cursor = con.cursor()
            Cursor.execute(sqlstr1+sqlstr2+');')
            #doing inserts
            values = [v.split('#')[0] for k, v in cfgitems]
#            for k, v in cfgitems:
#                v = v.split('#')[0]
#                if not v:
#                    v = ' '
#                values.append()
            str3 = ','.join(['"%s"'%i for i in values])+')'
            sqlstr3 = '''INSERT INTO %s VALUES(%s''' % (table, str3)
            #print sqlstr3
            Cursor.execute(sqlstr3)
        finally:
            if con:
                con.commit()
                con.close()

    def outToDb(self,table):
        """
        Insert simulation results on a mysql or SQLite table
        """

        if self.backend.lower() == "mysql":
            self.Say('Saving data on MySQL...')
        elif self.backend.lower() == "sqlite":
            self.Say('Saving data on SQLite...')

        con=None
        try:
            table = table+'_'+self.now
            self.writeMetaTable(table)
            self.outtable = table
            if self.backend.lower() == "mysql":
                con = MySQLdb.connect(host=self.host, port=self.port,
                                    user=self.usr,passwd=self.passw, db=self.db)
            elif self.backend.lower() == "sqlite":
                if not self.outdir == os.getcwd():
                    os.chdir(self.outdir)
                con = sqlite3.connect("Epigrass.sqlite")
                os.chdir(self.dir)
            # Define number of variables to be stored
            nvar = len(self.g.site_list[0].ts[-1]) +4 #state variables,  plus coords, plus incidence, plus infected arrivals.
            str1 = '`%s` FLOAT(9),'*nvar #nvar variables in the table
            str1lite = '%s REAL,'*nvar #nvar variables in the SQLite table
            varnames = ['lat','longit']+list(self.g.site_list[0].vnames)+['incidence']+['Arrivals']
#            print nvar, varnames, str1
            str1 = str1[:len(str1)-1] % tuple(varnames) #insert variable names (MySQL)
            str1lite = str1lite[:len(str1lite)-1] % tuple(varnames) #insert variable names (SQLITE)
            Cursor = con.cursor()
            str2 = """CREATE TABLE %s(
            `geocode` INT( 9 )  ,
            `time` INT( 9 ) ,
            `name` varchar(128) ,
            """ % table
            str2lite = """CREATE TABLE %s(
            geocode INTEGER  ,
            time INTEGER ,
            name TEXT ,
            """ % table
            sql = str2+str1+');'
            sqlite = str2lite+str1lite+');'

            if self.backend.lower() == "mysql":
                Cursor.execute(sql)
                str3 = (nvar+3)*'%s,'
                str3 = str3[:-1]+')'
            elif self.backend.lower() == "sqlite":
                Cursor.execute(sqlite)
                str3 = (nvar+3)*'?,'
                str3 = str3[:-1]+')'
            sql2 = 'INSERT INTO %s' % table + ' VALUES('+str3
            nvalues = []
            for site in self.g.site_list:
                geoc = site.geocode
                lat = site.pos[0]
                long = site.pos[1]
                name = site.sitename
                ts = array(site.ts[1:]) #remove init conds so that ts and inc are the same size
                inc = site.incidence
                thist = site.thetahist
                t = 0
                for i in ts:
                    tstep = str(t)
                    nvalues.append(tuple([geoc,tstep,name]+[lat,long]+list(i)+[inc[t]]+[thist[t]]))
                    t += 1
            Cursor.executemany(sql2,nvalues)
            #Creating a table for edge data
            self.etable = etable = table+'e'
            esql = """CREATE TABLE %s(
            `source_code` INT( 9 )  ,
            `dest_code` INT( 9 )  ,
            `time` INT( 9 ) ,
            `ftheta` FLOAT(9) ,
            `btheta` FLOAT(9) );"""%etable
            esqlite = """CREATE TABLE %s(
            source_code INTEGER  ,
            dest_code INTEGER  ,
            time INTEGER ,
            ftheta REAL ,
            btheta REAL );"""%etable

            if self.backend.lower() == "mysql":
                Cursor.execute(esql)
                esql2 = 'INSERT INTO %s' % etable + ' VALUES(%s,%s,%s,%s,%s)'
            elif self.backend.lower() == "sqlite":
                Cursor.execute(esqlite)
                esql2 = 'INSERT INTO %s' % etable + ' VALUES(?,?,?,?,?)'
            values = []
            for e in self.g.edge_list:
                s = e.source.geocode
                d = e.dest.geocode
                t=0
                for f,b in zip(e.ftheta,e.btheta):
                    values.append((s,d,t,f,b))
                    t += 1
            Cursor.executemany(esql2,values)


        finally:
            if con:
                con.close()
        #saving pickle of adjacency matrix
        matname = 'adj_'+self.modelName # table
        fname = os.path.join(self.outdir,matname)
        adjfile = open(fname,'w')
        cPickle.dump(self.g.getConnMatrix(),adjfile)
        adjfile.close()

    def criaAdjMatrix(self):
        # saving the adjacency  matrix
        codeslist = [str(i.geocode) for i in self.g.site_list]
        if not os.path.exists('adjmat.csv'):
            self.Say('Saving the adjacency  matrix...')
            am = self.g.getConnMatrix()
            amf = open ('adjmat.csv','w')
            amf.write(','.join(codeslist)+'\n')
            for row in am:
                row = [str(i) for i in row]
                amf.write(','.join(row)+'\n')
            amf.close()
            self.Say('Done!')

    def dumpData(self):
        """
        Dumps data as csv (comma-separated-values)
        """
##        pid = os.fork()
##        if pid:
##            return
##        else:
        self.Say("Starting simulation Analysis")
        curdir = os.getcwd()
        if not self.outdir == curdir:
            os.chdir(self.outdir)

        codeslist = [str(i.geocode) for i in self.g.site_list]
        self.criaAdjMatrix()
        #saving the shortest path matrices
#        if not os.path.exists('spmat.csv'):
#            self.Say('Calculating the shortest path matrices...')
#            ap = self.g.getAllPairs()
#            f = open('ap','w')
#            cPickle.dump(ap,f)
#            f.close()
#            spd = self.g.shortDistMatrix
#            apf = open ('spmat.csv','w')
#            apf.write(','.join(codeslist)+'\n')
#            spdf = open ('spdmat.csv','w')
#            spdf.write(','.join(codeslist)+'\n')
#            for row in ap:
#                row = [str(i) for i in row]
#                apf.write(','.join(row)+'\n')
#            apf.close()
#            for row in spd:
#                row = [str(i) for i in row]
#                spdf.write(','.join(row)+'\n')
#            spdf.close()
#            print 'Done!'

        #Saving epidemic path
        self.Say('Saving Epidemic path...')
        if self.round:
            epp = codecs.open('epipath%s.csv'%str(self.round),'w',self.encoding)
        else:
            epp = codecs.open('epipath.csv','w',self.encoding)
        epp.write('time,site,infector\n')
        for i in self.g.epipath:
            #print i
            infectors = i[-1]
            # sorting infectors by number of infective contributed
            if len(infectors):
                reverse_infectors = [ [v[1],v[0]] for v in infectors.items()]
                reverse_infectors.sort()
                mli = [reverse_infectors[j][1] for j in xrange(0,len(reverse_infectors))][-1].sitename#Most likely infector
            else:
                mli = 'NA'
            #print i[1].sitename, type(i[1].sitename), mli
            epp.write(str(i[0])+','+i[1].sitename+','+mli+'\n')
        epp.close()
        self.Say('Done!')

        #saving Epistats
        self.Say('Saving Epidemiological results...')
        stats = [str(i) for i in self.g.getEpistats()]
        seed = [s for s in self.g.site_list if s.geocode == self.seed[0][0]][0]
        stats.pop(1) #Remove epispeed which is a vector
        if os.path.exists('epistats.csv'):
            stf = codecs.open('epistats.csv','a',self.encoding)#append to file
        else:
            stf = codecs.open('epistats.csv','w',self.encoding)#create a new file
            stf.write('seed,name,size,infected_sites,spreadtime,median_survival,totvaccinated,totquarantined,seeddeg,seedpop\n')
##            stf.write('seed,name,size,infected_sites,spreadtime,median_survival,totvaccinated,totquarantined,seeddeg,seedpop\n')
        #scent = str(seed.getCentrality())
        #sbetw = str(seed.getBetweeness())
        #sthidx = str(seed.getThetaindex())
        sdeg = str(seed.getDegree())
        spop = str(seed.totpop)
        sname =seed.sitename
        sstats = '%s,%s'%(sdeg,spop)
#        sstats = '%s,%s,%s,%s,%s'%(scent,sbetw,sthidx,sdeg,spop)
        stf.write(str(self.seed[0][0])+','+sname+','+','.join(stats)+','+sstats+'\n')
        stf.close()
        self.Say('Done!')

        #Saving site stats
        self.Say('Saving site statistics...')
        #self.g.getAllPairs()
        if os.path.exists('sitestats.csv'):
            sitef = codecs.open('sitestats.csv','a',self.encoding) #append to file
        else:
            sitef = codecs.open('sitestats.csv','w',self.encoding)

        #sitef.write('round,geocode,name,infection_time,degree,centrality,betweeness,theta_index,distance,seed,seedname\n')
        sitef.write('round,geocode,name,infection_time,degree,seed,seedname\n')
        for s in self.g.site_list:
            degree = str(s.getDegree())
            #central = str(s.getCentrality())
            #bet = str(s.getBetweeness())
            #thidx = str(s.getThetaindex())
            seedgc = str(self.seed[0][0])
            seedname = seed.sitename
#            f = open('ap','r')
#            ap = cPickle.load(f)
#            f.close()
#            distseed = str(ap[codeslist.index(str(s.geocode)),codeslist.index(str(self.seed[0][0]))])
            it = str(s.infected) #infection time
            if it == 'FALSE':
                it = 'NA'

#            sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+central+','+bet+','+thidx+','+distseed+','+seedgc+','+seedname+'\n')
            sitef.write(str(self.round)+','+str(s.geocode)+','+s.sitename+','+it+','+degree+','+seedgc+','+seedname+'\n')

        os.chdir(curdir)

        self.Say('Done saving data!')

    def saveModel(self,fname):
        """
        Save the fully specified graph.
        """
        f = open(fname,'w')
        cPickle.dump(self.g,f)
        f.close()

    def loadModel(self,fname):
        """
        Loads a pre-saved graph.
        """
        g = cPickle.load(fname)
        return g

    def fins(self):
        """
        call parallel inserts
        """

    def runGraph(self,graphobj,iterations=1, transp=0):
        """
        Starts the simulation on a graph.
        """
        g = graphobj
        g.maxstep = iterations
        sites = tuple(graphobj.site_list)
        edges = tuple(graphobj.edge_list)
        #viewer = Viewer(g,self.shapefile[0], "geocode" )
        if transp:
            for n in xrange(iterations):
                for i in sites:
                    i.runModel()
                for j in edges:
                    j.transportStoD()
                    j.transportDtoS()
##                self.outToODb(self.modelName,mode='p')
                if self.gui:
#                    self.gui.graphDisplay.drawStep(g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
                    self.gui.stepLCD.display(g.simstep)
                    self.gui.app.processEvents()
#                    self.gui.RT.mutex.lock()
#                    self.gui.RT.emit(self.gui.QtCore.SIGNAL("drawStep"), g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
#                    self.gui.RT.mutex.unlock()
                g.simstep += 1
        else:
            for n in xrange(iterations):
                for i in sites:
                    i.runModel()
##                self.outToODb(self.modelName,mode='p')
                #viewer.show(n)
                if self.gui:
                    #self.gui.graphDisplay.drawStep(g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
                    self.gui.stepLCD.display(g.simstep)
                    self.gui.app.processEvents()
                    self.gui.RT.mutex.lock()
                    self.gui.RT.emit(self.gui.QtCore.SIGNAL("drawStep"), g.simstep, dict([(s.geocode, s.incidence[-1]) for s in sites]))
                    self.gui.RT.mutex.unlock()
                g.simstep += 1
    def Say(self,string):
        """
        Exits outputs messages to the console or the gui accordingly
        """
        if self.gui:
            self.gui.textEdit1.insertPlainText(string+'\n''')
        print string+'\n'
def storeSimulation(s,db='epigrass', host='localhost',port=3306):
    """
    store the Simulate object *s* in the epigrass database
    to allow distributed runs. Currently not working.
    """
    now = time.asctime().replace(' ','_').replace(':','')
    table = 'Model_'+S.modelName+now
    con = MySQLdb.connect(host=host, port=port, user=usr,passwd=passw, db=db)
    Cursor = con.cursor()
    sql = """CREATE TABLE %s(
        `simulation` BLOB);""" % table
    Cursor.execute(sql)
    blob = cPickle.dumps(s)
    sql2 = 'INSERT INTO %s' % table + ' VALUES(%s)'
    Cursor.execute(sql2,blob)
    con.close()

class Bunch:
    """
    Class to store a bunch os variables
    """
    def __init__(self, **kw):
       self.__dict__.update(kw)

class Tree:
    """
    Tree object representing the spread of an epidemic
    """
    def __init__(self,Simulation):
        self.g = Simulation.g
        Self.S = Simulation
        pass

    def writeNexus(self):
        """
        Writes a Nexus file containing(.nex) the phylogeographical
        tree of the epidemic.
        """
        ntax = len(self.g.epipath)
        sitenames = [i[1].sitename for i in self.g.epipath]
        Name = self.S.modelName
        header = """#NEXUS
        [written %s from model % by EpiGrass]
        """%(time.asctime(),Name)
        taxablock = """BEGIN TAXA;
                            TITLE %s;
                            DIMENSIONS NTAX=%s;
                            TAXLABELS
                                %s
                                ;
                        END;

        """%(Name,ntax,' '.join(sitenames))

        treeblock = """BEGIN TREES;
                        Title Epidemic_Path;
                        LINK Taxa = %s;"""%(Name,)

        transtring = ','.join(['%s %s'%(i,j) for i,j in zip(range(1,len(sitenames)+1),sitenames)])+';'
        treeblock2 = """TRANSLATE
                            %s
                            """%transtring
        venn = self.getVenn()
        treeblock3 = """TREE 'Default ladder+' = %;
                        END;"""%venn
    def getVenn(self):
        """"
        Generates Venn Diagram from epipath.
        """
        pass

def onStraightRun(options, args):
    """
    Runs the model from the commandline
    """
    if options.backend =="mysql":
        S = simulate(fname=args[0],host=options.dbhost, user=options.dbuser, password=options.dbpass,  backend=options.backend)
    else:
        S = simulate(fname=args[0],backend=options.backend)
    if not S.replicas:
        S.start()
        spread.Spread(S.g)
    else:
        repRuns(S)

    if S.Batch:
        self.Say('Simulation Started.')

        # run the batch list
        for i in S.Batch:
            #Makes sure it comes back to original directory before opening models in the batch list
            os.chdir(S.dir)
            #delete the old graph object to save memory
            S.graph = None
            # Generates the simulation object
            T = simulate(fname=i,host=S.host, user=S.usr, password=S.passw, backend=S.backend)

            self.Say('starting model %s'%i)
            T.start()  # Start the simulation
#            spread.Spread(T.g)

def repRuns(S):
        """
        Do repeated runs
        """
        randseed=S.randomize_seeds
        fname=S.fname
        host=S.host
        user=S.usr
        password=S.passw
        backend=S.backend
        nseeds= S.seed[0][2] #number o individual to be used as seeds
        self.Say("Replication type: ", randseed)
        if randseed:
            seeds = S.randomizeSeed(randseed)
        reps = S.replicas
        for i in xrange(reps):
            self.Say("Starting replicate number %s"%i)
            S = simulate(fname=fname,host=host, user=user, password=password, backend=backend)
            if randseed:
                S.setSeed(seeds[i],nseeds)
            S.round = i
            S.shpout = False
            S.start()
            del S


def main():
    # Options and Argument parsing for running model from the command line, without the GUI.
    usage = "usage: %prog [options] your_model.epg"
    parser = OptionParser(usage=usage, version="%prog "+__version__.version)
    parser.add_option("-b", "--backend",type="string",  dest="backend",
                  help="Define which datastorage backend to use", metavar="<mysql|sqlite|csv>",  default="sqlite")
    parser.add_option("-u", "--dbusername", type="string",
                  dest="dbuser", help="MySQL user name")
    parser.add_option("-p", "--password", type="string",
                  dest="dbpass", help="MySQL password for user")
    parser.add_option("-H", "--dbhost", type="string",
                  dest="dbhost",default="localhost",  help="MySQL hostname or IP address")

    (options, args) = parser.parse_args()
    if options.backend == "mysql" and not (options.dbuser and options.dbpass):
        parser.error("You must specify a user and password when using MySQL.")
    if options.backend not in ['mysql', 'sqlite', 'csv']:
        parser.error('"%s" is an invalid backend type.'%options.backend)

    if args:
        if len(args)<1:
            parser.error("You must provide an EPG file to run.")
        elif len(args)<1:
            parser.error("Only a single EPG file can be specified.")
        else:
            if not os.path.exists(args[0]):
                parser.error("The file '%s' does not exist."%args[0])
    else:
        parser.error("Wrong number of arguments.")
    onStraightRun(options, args)
if __name__ == '__main__':
    main()
    







# This is a custom model to used in place of Epigrass' built-in models. Custom
# models must always be on a file named CustomModel.py and contain at least 
# a function named Model. Both the File name and the function Names are case-sensitive,
# so be careful. Please refer to the manual for intructions on how to write your 
# own custom models.

def Model(self,vars,par,theta=0, npass=0):
        """
        This function implements the SIR model
        - inits = (E,I,S)
        - par = (Beta, alpha, E,r,delta,B, w, p) see docs.
        - theta = infectious individuals from neighbor sites
        """
        ##### Defining variable names to appear in the database
        # Must be listed in the same order of variables in the model 
        self.parentSite.vnames = ('Exposed','Infectious','Susceptible')
        
        ##### Get state variables' current values
        if self.parentSite.parentGraph.simstep == 1: #get initial values
            E,I,S = (self.bi['e'],self.bi['i'],self.bi['s'])
        else: # get last value
            E,I,S = vars
            
        ##### Defining N, the total population     
        N = self.parentSite.totpop
        
        ##### Getting values for the model parameters
        beta,alpha,e,r,delta,B,w,p = (self.bp['beta'],self.bp['alpha'],self.bp['e'],self.bp['r'],self.bp['delta'],self.bp['b'],self.bp['w'],self.bp['p'])
        
        ##### Defining a Vacination event (optional)
        if self.parentSite.vaccineNow:
            S -= self.parentSite.vaccov*S
        
        ##### Modeling the number of new cases (incidence function)
        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
        
        ##### Epidemiological model (SIR)
        Ipos = (1-r)*I + Lpos
        Spos = S + B - Lpos
        Rpos = N-(Spos+Ipos)
        
        ##### Updating stats
        self.parentSite.incidence.append(Lpos) # incidence time series 
        self.parentSite.totalcases += Lpos  # total cases so far
        
        #### If parentSite was not infected before and now it is... 
        if not self.parentSite.infected: 
            if Lpos > 0:
                #if not self.parentSite.infected:
                self.parentSite.infected = self.parentSite.parentGraph.simstep   # store the time of first infection
                self.parentSite.parentGraph.epipath.append((self.parentSite.parentGraph.simstep,self.parentSite,self.parentSite.infector))
        # Store the number of infectious individuals commuting to parentSite.
        self.parentSite.migInf.append(Ipos)
        
        return [0,Ipos,Spos]



#
#spread display and analisys
#
#try:
#    from PyQt4.QtGui import *
#except ImportError: 
#    print "Please install PyQT 4"
#from qt import *
from xml.dom import minidom, Node
import os, string
#import visual as V
from math import *
from numpy import *
import codecs

class Spread:
    def __init__(self, graphobj, outdir='.',encoding='latin-1'):
        self.g = graphobj
        self.outdir = outdir
        self.encoding = encoding
#        ct = self.cleanTree()
        #self.dotDraw(ct)
#        self.writeGML(ct,outdir,encoding)
        graphml = GraphML(self.g, outdir, encoding)
        graphml.write()
    
    def cleanTree(self):
        """
        Generates a unambiguous spread tree by selecting the most likely infector for each site
        """
        sptree=[]
        for i in self.g.epipath:
            #print i
            infectors = i[-1]
            # sorting infectors by number of infective contributed
            if len(infectors):
                reverse_infectors = [ [v[1],v[0]] for v in infectors.items()]
                reverse_infectors.sort()
                mli = [reverse_infectors[j][1] for j in xrange(0,len(reverse_infectors))][-1]#Most likely infector
            else:
                mli = 'NA'
            sptree.append((i[0],i[1].sitename,mli))
        return sptree
    
    def dotDraw(self,tr):
        """
        generate a jpeg image of the spread tree using pydot
        """
        edges = []
        for i in tr:
            edges.append((i[1],i[2]))
        dotg=pydot.graph_from_edges(edges)
        for e in tr: # label edges with date of infection
            dotg.get_edge(e[1],e[2]).label = str(e[0])
        dotg.write_raw('spreadtree.dot')
        #dotg.write_png('graph.jpg',prog='dot')
    
    
    def display(self):
        """
        display the epidemic tree
        """
        pass
        
        
    def writeGML(self,tree, outdir,encoding,fname="spreadtree.gml"):
        """
        Save the tree in the GML format
        """
        try:
            os.chdir(outdir)
        except:
            pass
        dir(self)
        f = codecs.open(fname,'w', encoding)
        f.writelines(['Creator "Epigrass"\n',
        'Version ""\n',
        'graph\n[\n',
        '\thierarchic\t1\n'
        '\tlabel\t"Spread Tree"\n'
        '\tdirected\t1\n'])
        #self.writeENGML(f,tree)
        Spread.writeENGML(f,tree) #calling as a class method
        f.write(']')
        f.close()
        print "Wrote %s"%fname
    writeGML = classmethod(writeGML)
        
    def writeENGML(self,fobj,tree):
        """
        Write the edges and Nodes section of a GML file
        """
        f=fobj
        #Create dictionary of node IDs, and eliminate possible node duplicates.
        nodes = dict([(i[1],n) for n,i in enumerate(tree)])
        for n,k in enumerate(nodes.iterkeys()):
            nodes[k] = n
        #writing nodes
        for i,n in nodes.iteritems():
            f.writelines(['\tnode\n','\t[\n'])
            f.writelines(['\t\tid\t%s\n'%n,'\t\tlabel\t"%s"\n'%i])
            f.writelines(['\t\tgraphics\n','\t\t[\n','\t\t\tw\t60\n','\t\t\th\t30\n'])
            f.writelines(['\t\t\ttype\t"roundrectangle"\n','\t\t\tfill\t"#FFCC00"\n','\t\t\toutline\t"#000000"\n','\t\t]\n'])
            f.writelines(['\t\tLabelGraphics\n','\t\t[\n','\t\t\ttext\t"%s"\n'%i,'\t\t\tfontSize\t13\n','\t\t\tfontName\t"Dialog"\n','\t\t\tanchor\t"c"\n','\t\t]\n','\t]\n'])
        #writing Edges
        for n,i in enumerate(tree):
            lab = str(i[0])
            tid = nodes[i[1]]
            try: #If the source is NA (seed site)
                sid = nodes[i[2]]
            except KeyError:
                continue
            #print lab
            f.writelines(['\tedge\n','\t[\n'])
            f.writelines(['\t\tsource\t%s\n'%sid,'\t\ttarget\t%s\n'%tid,'\t\tlabel\t"%s"\n'%lab,'\t\tgraphics\n','\t\t[\n'])
            f.writelines(['\t\t\tfill\t"#000000"\n','\t\t\ttargetArrow\t"standard"\n','\t\t]\n','\t]\n'])
    writeENGML = classmethod(writeENGML)

class GraphML:
    def __init__(self,graphobj,  outdir, encoding,fname="spread.graphml" ):
        """
        Generates a valid GraphML document from the spread tree.
        """
        self.g = graphobj
        self.encoding = encoding
        self.outdir = outdir
        self.fname = fname
        self.doc = minidom.Document()
        # Creating Root element
        gml = self.doc.createElement("graphml")
        gml.setAttribute("xmlns","http://graphml.graphdrawing.org/xmlns")
        gml.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        gml.setAttribute("xsi:schemaLocation", "http://graphml.graphdrawing.org/xmlns http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd")
        self.addAttrKeys(gml)
        
        # Creating graph Element
        self.gr = self.doc.createElement("graph")
        self.gr.setAttribute("id", "graphname")
        self.gr.setAttribute("edgedefault", "directed")
        
        #Creating nodes and edges
        self.nodes = []
        for n in self.g.epipath:
            infectors = n[-1]
            self.addNodeEl(n[1].geocode, n[1].sitename)
            self.nodes.append(n[1].geocode)
            for  i, c in infectors.iteritems():
                self.addNodeEl(i.geocode, i.sitename)
                self.addEdgeEl(i.geocode, n[1].geocode, n[0], c)
        gml.appendChild(self.gr)
        self.doc.appendChild(gml)
                
    def addNodeEl(self, gc , name):
        """
        Adds a Node element to the Graphml object
        """
        if gc in self.nodes:
            return
        else:
#            print gc, name
            nd = self.doc.createElement("node")
            nd.setAttribute("id", "n"+str(gc))
            data = self.doc.createElement("data")
            data.setAttribute("key", "d0")
            data.appendChild(self.doc.createTextNode(name))
            nd.appendChild(data)
            self.gr.appendChild(nd)
            
    def addEdgeEl(self, s, d , t, ino):
#        print s, d , t , ino
        ed = self.doc.createElement("edge")
        ed.setAttribute("source", str(s))
        ed.setAttribute("target", str(d))
        data = self.doc.createElement("data")
        data.setAttribute("key", "d1")
        data.appendChild(self.doc.createTextNode(str(ino)))
        ed.appendChild(data)
        data = self.doc.createElement("data")
        data.setAttribute("key", "d2")
        data.appendChild(self.doc.createTextNode(str(t)))
        ed.appendChild(data)
        self.gr.appendChild(ed)
    
    def addAttrKeys(self, gml):
        k1 = self.doc.createElement("key")
        k1.setAttribute("id", "d0")
        k1.setAttribute("for", "node")
        k1.setAttribute("attr.name", "label")
        k1.setAttribute("attr.type", "string")
        gml.appendChild(k1)
        k2 = self.doc.createElement("key")
        k2.setAttribute("id", "d1")
        k2.setAttribute("for", "edge")
        k2.setAttribute("attr.name", "Innoculum")
        k2.setAttribute("attr.type", "int")
        gml.appendChild(k2)
        k3 = self.doc.createElement("key")
        k3.setAttribute("id", "d2")
        k3.setAttribute("for", "edge")
        k3.setAttribute("attr.name", "timestep")
        k3.setAttribute("attr.type", "int")
        gml.appendChild(k3)
    def write(self):
        """
        Writes the graphml file to disk
        """
        fullpath = os.path.join(self.outdir,self.fname)
        f=open(fullpath,"w")
#        f.write('<?xml version="1.0" encoding="%s"?>'%self.encoding)
        f.write(self.doc.toprettyxml(encoding=self.encoding))
        f.close()
        
class Consensus:
    def __init__(self,path,cutoff=0.0):
        tl = self.readTress(path)
        self.consensus(tl,cutoff)
    def readTress(self,path):
        """
        Read all files named epipath* from the current dir
        and return a collection of trees.
        """
        if not os.path.exists(path+'epipath.csv'):
            print "No tree files available on this path"
        else:
            f = open(path+'epipath.csv','r')
            treelist = [self.parseEpipath(f.readlines())]
            f.close()
            n=1
            fname = path+'epipath'+str(n)+'.csv'
            while os.path.exists(fname):
                f= open(fname, 'r')
                print "Reading %s ..." % fname
                treelist.append(self.parseEpipath(f.readlines()))
                f.close()
                n+=1
                fname = path+'epipath'+str(n)+'.csv'
        return treelist
        
        
    def parseEpipath(self,lines):
        """
        Receives a list of strings and returns a list of tuples
        """
        tree = [tuple(l[:-1].split(',')) for l in lines]
        return tree
        
    
    def consensus(self, treelist, cutoff):
        """
        Generate a consensus tree from the various trees generated by multiple runs.
        Saves the tree in gml format file
        """
        cons = {}
        trees = []
        for t in treelist:
            trees += t #concatenate all the trees
##        import tree 
##        cons = tree.tree(trees)
        #print len(trees)
        for i in trees[1:]:
            try:
                cons[i[1]+'-'+i[2]] += 1
            except:
                try:
                    cons[i[2]+'-'+i[1]] += 1
                except:
                    cons[i[1]+'-'+i[2]] = 1
        #print len(cons)
        
        maxsup = max(cons.values())
        const = [tuple([v]+k.split('-')) for k,v in cons.items()if float(v)/maxsup > cutoff[0]/100.]
        Spread.writeGML(const,'.','latin-1','consensus_tree.gml')
                 



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/fccoelho/Documents/Projects_software/epigrass/epigrassqt4/Epigrass/neteditor.ui'
#
# Created: Tue Aug 11 14:10:16 2009
#      by: PyQt4 UI code generator 4.4.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(759, 588)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/egicon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralWidget = QtGui.QWidget(MainWindow)
        self.centralWidget.setObjectName("centralWidget")
        self.verticalLayout = QtGui.QVBoxLayout(self.centralWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtGui.QSplitter(self.centralWidget)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.graphicsView = QtGui.QGraphicsView(self.splitter)
        self.graphicsView.setObjectName("graphicsView")
        self.dataPanel = QtGui.QTabWidget(self.splitter)
        self.dataPanel.setObjectName("dataPanel")
        self.Nodes = QtGui.QWidget()
        self.Nodes.setObjectName("Nodes")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.Nodes)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.nodeTable = QtGui.QTableWidget(self.Nodes)
        self.nodeTable.setAlternatingRowColors(True)
        self.nodeTable.setObjectName("nodeTable")
        self.nodeTable.setColumnCount(0)
        self.nodeTable.setRowCount(0)
        self.verticalLayout_2.addWidget(self.nodeTable)
        self.dataPanel.addTab(self.Nodes, "")
        self.Edges = QtGui.QWidget()
        self.Edges.setObjectName("Edges")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.Edges)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.edgeTable = QtGui.QTableWidget(self.Edges)
        self.edgeTable.setAlternatingRowColors(True)
        self.edgeTable.setObjectName("edgeTable")
        self.edgeTable.setColumnCount(0)
        self.edgeTable.setRowCount(0)
        self.verticalLayout_3.addWidget(self.edgeTable)
        self.dataPanel.addTab(self.Edges, "")
        self.verticalLayout.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.centralWidget)
        self.menuBar = QtGui.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 759, 25))
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtGui.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        self.menu_Export = QtGui.QMenu(self.menuFile)
        self.menu_Export.setObjectName("menu_Export")
        self.menu_View = QtGui.QMenu(self.menuBar)
        self.menu_View.setObjectName("menu_View")
        self.menuAnalyze = QtGui.QMenu(self.menuBar)
        self.menuAnalyze.setObjectName("menuAnalyze")
        self.menu_Debug = QtGui.QMenu(self.menuAnalyze)
        self.menu_Debug.setObjectName("menu_Debug")
        MainWindow.setMenuBar(self.menuBar)
        self.statusBar = QtGui.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.toolBar = QtGui.QToolBar(MainWindow)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionOpen_edges_file = QtGui.QAction(MainWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/neteditor/openedges.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionOpen_edges_file.setIcon(icon1)
        self.actionOpen_edges_file.setObjectName("actionOpen_edges_file")
        self.actionOpen_Shapefile = QtGui.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/neteditor/openshape.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionOpen_Shapefile.setIcon(icon2)
        self.actionOpen_Shapefile.setObjectName("actionOpen_Shapefile")
        self.action_Save = QtGui.QAction(MainWindow)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/neteditor/document-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.action_Save.setIcon(icon3)
        self.action_Save.setObjectName("action_Save")
        self.actionSave_as = QtGui.QAction(MainWindow)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/neteditor/document-save-as.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSave_as.setIcon(icon4)
        self.actionSave_as.setObjectName("actionSave_as")
        self.action_Graph = QtGui.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/neteditor/graph.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.action_Graph.setIcon(icon5)
        self.action_Graph.setObjectName("action_Graph")
        self.actionE_xit = QtGui.QAction(MainWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/neteditor/application-exit.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionE_xit.setIcon(icon6)
        self.actionE_xit.setObjectName("actionE_xit")
        self.actionStats = QtGui.QAction(MainWindow)
        self.actionStats.setObjectName("actionStats")
        self.action_Close = QtGui.QAction(MainWindow)
        self.action_Close.setObjectName("action_Close")
        self.actionCheck_Graph = QtGui.QAction(MainWindow)
        self.actionCheck_Graph.setObjectName("actionCheck_Graph")
        self.actionDot = QtGui.QAction(MainWindow)
        self.actionDot.setObjectName("actionDot")
        self.actionGML = QtGui.QAction(MainWindow)
        self.actionGML.setObjectName("actionGML")
        self.actionPajek = QtGui.QAction(MainWindow)
        self.actionPajek.setObjectName("actionPajek")
        self.actionYAML = QtGui.QAction(MainWindow)
        self.actionYAML.setObjectName("actionYAML")
        self.menu_Export.addAction(self.actionDot)
        self.menu_Export.addAction(self.actionGML)
        self.menu_Export.addAction(self.actionPajek)
        self.menu_Export.addAction(self.actionYAML)
        self.menuFile.addAction(self.actionOpen_edges_file)
        self.menuFile.addAction(self.actionOpen_Shapefile)
        self.menuFile.addAction(self.action_Close)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_Save)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addAction(self.menu_Export.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionE_xit)
        self.menu_View.addAction(self.action_Graph)
        self.menu_Debug.addAction(self.actionCheck_Graph)
        self.menuAnalyze.addAction(self.menu_Debug.menuAction())
        self.menuAnalyze.addSeparator()
        self.menuAnalyze.addAction(self.actionStats)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menu_View.menuAction())
        self.menuBar.addAction(self.menuAnalyze.menuAction())
        self.toolBar.addAction(self.actionOpen_edges_file)
        self.toolBar.addAction(self.actionOpen_Shapefile)
        self.toolBar.addAction(self.action_Save)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.action_Graph)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionE_xit)

        self.retranslateUi(MainWindow)
        self.dataPanel.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Network Editor", None, QtGui.QApplication.UnicodeUTF8))
        self.graphicsView.setStatusTip(QtGui.QApplication.translate("MainWindow", "Map visualization window", None, QtGui.QApplication.UnicodeUTF8))
        self.Nodes.setToolTip(QtGui.QApplication.translate("MainWindow", "Node set", None, QtGui.QApplication.UnicodeUTF8))
        self.nodeTable.setStatusTip(QtGui.QApplication.translate("MainWindow", "Node list.", None, QtGui.QApplication.UnicodeUTF8))
        self.nodeTable.setSortingEnabled(False)
        self.dataPanel.setTabText(self.dataPanel.indexOf(self.Nodes), QtGui.QApplication.translate("MainWindow", "Nodes", None, QtGui.QApplication.UnicodeUTF8))
        self.edgeTable.setStatusTip(QtGui.QApplication.translate("MainWindow", "List of edges", None, QtGui.QApplication.UnicodeUTF8))
        self.edgeTable.setSortingEnabled(False)
        self.dataPanel.setTabText(self.dataPanel.indexOf(self.Edges), QtGui.QApplication.translate("MainWindow", "Edges", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Export.setTitle(QtGui.QApplication.translate("MainWindow", "&Export", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_View.setTitle(QtGui.QApplication.translate("MainWindow", "&View", None, QtGui.QApplication.UnicodeUTF8))
        self.menuAnalyze.setTitle(QtGui.QApplication.translate("MainWindow", "&Analyze", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Debug.setTitle(QtGui.QApplication.translate("MainWindow", "&Debug", None, QtGui.QApplication.UnicodeUTF8))
        self.toolBar.setWindowTitle(QtGui.QApplication.translate("MainWindow", "toolBar", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_edges_file.setText(QtGui.QApplication.translate("MainWindow", "Open &edges file", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_Shapefile.setText(QtGui.QApplication.translate("MainWindow", "Open &Shapefile", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Save.setText(QtGui.QApplication.translate("MainWindow", "&Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as.setText(QtGui.QApplication.translate("MainWindow", "Save &as", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Graph.setText(QtGui.QApplication.translate("MainWindow", "&Graph", None, QtGui.QApplication.UnicodeUTF8))
        self.actionE_xit.setText(QtGui.QApplication.translate("MainWindow", "E&xit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionStats.setText(QtGui.QApplication.translate("MainWindow", "Stats", None, QtGui.QApplication.UnicodeUTF8))
        self.action_Close.setText(QtGui.QApplication.translate("MainWindow", "&Close", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCheck_Graph.setText(QtGui.QApplication.translate("MainWindow", "Check Graph", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDot.setText(QtGui.QApplication.translate("MainWindow", "Dot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionGML.setText(QtGui.QApplication.translate("MainWindow", "GML", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPajek.setText(QtGui.QApplication.translate("MainWindow", "Pajek", None, QtGui.QApplication.UnicodeUTF8))
        self.actionYAML.setText(QtGui.QApplication.translate("MainWindow", "YAML", None, QtGui.QApplication.UnicodeUTF8))

import epigrass_rc

if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    MainWindow = QtGui.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from PyQt4 import QtCore, QtGui
#    from PyQt4.QtGui import *
except ImportError:
    print "Please install PyQT 4"
from Epigrass.Ui_epgeditor import Ui_MainWindow
from Epigrass.Ui_about4 import Ui_aboutDialog as aboutDialog
import ConfigParser
import StringIO
import epigrass_rc

class Editor(QtGui.QMainWindow,Ui_MainWindow):
    def __init__(self, argv):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)
        self.cp = ConfigParser.SafeConfigParser()
        if len(argv) <2:
            self.fname = ''
        else:
            self.fname = argv[1]
            self.fillTree(self.fname)
        QtCore.QObject.connect(self.exitButton,QtCore.SIGNAL("clicked()"),self.close)
        self.editing = None
        self.helpBrowser.setSource(QtCore.QUrl("qrc:/HelpEpg.html"))
       
        

    def checkScript(self):
        """
        Checks the loaded script for required headers and variables
        """
        self.fixed = sections = {'TRANSPORTATION MODEL':{'stochastic':0, 'speed':0, 'dotransp':1},
        'THE WORLD':{'sites':'', 'shapefile':'', 'encoding':'', 'edges':''},
        'SIMULATION AND OUTPUT':{'steps':50, 'siterep':[], 'report':0, 'replicas':0, 'randseed':0, 'outdir':'', 'mysqlout':0, 'batch':[]},
        'MODEL PARAMETERS':{'w':0, 'r':1, 'p':0, 'e':1, 'delta':0, 'beta':0, 'b':0, 'alpha':1},
        'INITIAL CONDITIONS':{'s':'N', 'i':0, 'e':0},
        'EPIDEMIOLOGICAL MODEL':{'modtype':'SIR'},
        'EPIDEMIC EVENTS':{'vaccinate':[], 'seed':[], 'quarantine':[]}}
        for i in sections.items():
            try:
                assert (self.cp.has_section(i[0]))
                for v in i[1].items():
                    try:
                        self.cp.get(i[0], v[0])
                    except:
                        QtGui.QMessageBox.information(None,
                            self.trUtf8("Required variable missing"),
                            self.trUtf8("""Variable %s was missing. Variable created and value set to default"""%v))
                        self.cp.set(i[0], v[0],str(v[1]))
            except AssertionError:
                    QtGui.QMessageBox.information(None,
                    self.trUtf8("Required section missing"),
                    self.trUtf8("""Section %s was missing. It was added to your file with default values for its variables."""%i[0]))
                    self.cp.add_section(i[0])
                    for j in i[1].items():
                        self.cp.set (i[0],j[0] , str(j[1]))
        

    def fillTree(self, fname):
        """
        Fills the tree from the configuration file's
        contents. 
        Returns a ConfigParser object
        """
        self.treeWidget.clear()
        self.cp.read(fname)
        self.checkScript()
        self.updateViewer()
        for sec in self.cp.sections():
            topitem = QtGui.QTreeWidgetItem(self.treeWidget)
            topitem.setText(0, sec)
            topitem.setText(1, "---")
            topitem.setText(2, "Section Header")
            self.treeWidget.addTopLevelItem(topitem)
            for opt in self.cp.options(sec):
                chitem = QtGui.QTreeWidgetItem(topitem)
                chitem.setText(0, opt)
                pos = self.cp.get(sec,opt).find('#')
                if pos >=0:
                    value = self.cp.get(sec,opt)[:pos]
                    comment = self.cp.get(sec,opt)[pos:]
                else:
                    value = self.cp.get(sec,opt)
                    comment = "variable"
                chitem.setText(1, value)
                chitem.setText(2, comment)

    def updateViewer(self):
        vf = StringIO.StringIO()
        self.cp.write(vf)
        self.epgView.setPlainText(vf.getvalue())
        vf.close()

    @QtCore.pyqtSignature("")
    def on_action_About_activated(self):
        QtGui.QMessageBox.about(None,
            self.trUtf8("About .EPG Editor"),
            self.trUtf8("""This editor is for creating or modifying .epg files."""))

    @QtCore.pyqtSignature("")
    def on_actionAbout_Epigrass_activated(self):
        aboutD = QtGui.QDialog()
        ab = aboutDialog()
        ab.setupUi(aboutD)
        aboutD.show()
        aboutD.exec_()

    @QtCore.pyqtSignature("")
    def on_action_Open_epg_File_activated(self):
#        print "clicked"
        fname = str(QtGui.QFileDialog.getOpenFileName(\
            None,
            self.trUtf8("Open .epg File"),
            QtCore.QString(),
            self.trUtf8("*.epg"),
            None))
        if fname:
            self.fillTree(fname)
        self.fname = fname

    @QtCore.pyqtSignature("")
    def on_action_New_activated(self):
        fname =  QtGui.QInputDialog.getText(\
            None,
            self.trUtf8("File Name"),
            self.trUtf8("Enter a name for you new .epg file:"),
            QtGui.QLineEdit.Normal)
        print fname
        if fname[1]:
            fname = str(fname[0])
            if not fname.endswith('.epg'):
                fname+='.epg'
            self.fillTree(fname)


    @QtCore.pyqtSignature("")
    def on_addVButton_clicked(self):
        """
        Add a new variable to the current section
        """
        item = QtGui.QInputDialog.getItem(\
            None,
            self.trUtf8("Choose Section"),
            self.trUtf8("Select a section to which you want to add a variable:"),
            self.fixed.keys(),
            0, True)
        if item[1]:
            sec = str(item[0])
            resp = QtGui.QInputDialog.getText(\
                None,
                self.trUtf8("Variable Name"),
                self.trUtf8("Enter the name of the variable to be created"),
                QtGui.QLineEdit.Normal)
            if resp[1]:
                nam = str(resp[0])
                self.cp.set(sec, nam, '')
                self.updateViewer()
#                print sec, type(sec)
                secit = self.treeWidget.findItems(sec, QtCore.Qt.MatchExactly, 0)[0]
                child = QtGui.QTreeWidgetItem(secit)
                child.setText(0, nam)
                secit.addChild(child)
                self.treeWidget.setCurrentItem(child)
                self.treeWidget.scrollToItem(child)
                self.on_treeWidget_itemDoubleClicked(child, 1)
#                print sec,  nam

    @QtCore.pyqtSignature("")
    def on_action_Help_activated(self):
        self.tabWidget.setCurrentIndex(2)

    @QtCore.pyqtSignature("")
    def on_action_Exit_activated(self):
        self.close()

    @QtCore.pyqtSignature("")
    def on_action_Save_activated(self):
        if not self.fname:
            self.fname = str(QtGui.QFileDialog.getSaveFileName(\
                None,
                self.trUtf8("Save File as"),
                self.fname,
                self.trUtf8("*.epg"),
                None)
)

        fo = open(self.fname, 'w')
        self.cp.write(fo)
        fo.close()



    @QtCore.pyqtSignature("QTreeWidgetItem *, int")
    def on_treeWidget_itemDoubleClicked(self, it, col):
#        print it, col
        if not it.parent():
            return #cannot edit sections
        else:
            if col == 0 and (str(it.text(0)) in self.fixed[str(it.parent().text(0))]):
                return#cannot edit required variables names
            self.editing=(it, col)
            self.treeWidget.openPersistentEditor(it, col)

    @QtCore.pyqtSignature("")
    def on_treeWidget_itemSelectionChanged(self):
        if self.editing:
            if str(self.editing[0].text(2)).startswith('#'): #Keep comments if they exist.
                self.cp.set(str(self.editing[0].parent().text(0)),str(self.editing[0].text(0)) , str(self.editing[0].text(1))+'\t'+str(self.editing[0].text(2)))
            else:
                self.cp.set(str(self.editing[0].parent().text(0)),str(self.editing[0].text(0)) , str(self.editing[0].text(1)))
            self.treeWidget.closePersistentEditor(*self.editing)
            self.editing = None
            self.on_action_Save_activated()
            self.updateViewer()
def main():
    import sys
    app = QtGui.QApplication(sys.argv)
    Form = Editor(sys.argv)
    Form.show()
    sys.exit(app.exec_())
    
if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Implementing 2D drawing primitives using
pyglet.gl
copyright 2007 by Flavio Codeco Coelho<fccoelho@gmail.com>
license: GPL v3
"""
from pyglet import font
from pyglet import clock
from pyglet import window
from pyglet import image
from pyglet.gl import *
from pyglet.window import mouse
from pyglet.window import event
from pyglet.window import key
import math,  ctypes

class Base(object):
    """
    Basic attributes of all drawing primitives
    """
    def __init__(self, x, y, z=0, color=(0.,0.,0.,1.), stroke=0, rotation=0):
        try :
            self.rect
        except AttributeError:
            self.rect = Rect(x,y,1,1) # this inits x,y and loc as well
        self.visible = 1 #
        self.z = z 
        self.rotation = rotation
        self.stroke = stroke
        self.color = color
        self.q = gluNewQuadric()
    def setLoc(self, p) : self.rect.loc = p 
    def getLoc(self) : return self.rect.loc
    def setX(self, x) : self.rect.x = x 
    def getX(self) : return self.rect.x 
    def setY(self, y) : self.rect.y = y 
    def getY(self) : return self.rect.y
    loc = property(getLoc, setLoc)
    x = property(getX, setX)
    y = property(getY, setY)
    def setWidth(self,w) : self.rect.width = w
    def getWidth(self) : return self.rect.width
    def setHeight(self, h) : self.rect.height = h
    def getHeight(self) : return self.rect.height
    width = property(getWidth, setWidth)
    height = property(getHeight, setHeight)
        
class Group:
    '''
    Group of objects which can be manipulated together
    '''
    def __init__(self, olist):
        for o in olist:
            if not isinstance(o, Base):
                print "objects included in list must all be drawings."
                return
        self.members = olist
    def getX(self) : 
       return sum([o.x for o in self.members] )/float(len(self.members))
    x  = property(fget= getX)
    def getY(self) : 
        return sum([o.y for o in self.members] )/float(len(self.members))
    y  = property(fget= getY)

    
    def move(self, dx=0, dy=0):
        '''
        Translate the entire group the specified distance in each of the axis.
        '''
        for o in self.members:
            o.setX(o.x+dx)
            o.setY(o.y+dy)

    def rotate(self, angle):
        """
        Rotate all members around the group's center of mass
        positive angle rotates counterclockwise
        """
        # TODO: fix bug that is making objects rotate in a spiral path instead of a circular one
        for o in self.members:
            dist = math.sqrt((o.x-self.x)**2.+(o.y-self.y)**2.)
            #angle from horizontal
            afh = math.asin((o.y-self.y)/(dist))
            if o.x <= self.x:
                afh += (math.pi/2.-afh)*2. if (o.y >= self.y) else-(math.pi/2.+afh)*2.
            #dx = dist*math.cos(afh)-dist*math.cos(afh+angle)
            #dy = dist*math.sin(angle+afh)-dist*math.sin(afh)
            x = self.x+math.cos(afh+angle)*dist
            y = self.y+math.sin(afh+angle)*dist
            #print afh+angle, type(o),  x
            o.setX(x)#(o.x+dx)
            o.setY(y)#(o.y+dy)
        pass

class Pixel(Base):
    """ A pixel at a given x,y,z position and color.
        Pixel(x=12, y=100, z=900, color=(1,0,0,0.5))
    """
    def render(self):
        """
            Draws a pixel at a given x and y with given color .
            Color = 3 or 4 arg tuple. RGB values from 0 to 1 being 1 max value (1, 1, 1) would be white
        """
        glColor4f(*self.color)
##        glDisable(GL_TEXTURE_2D) # disable in case it was on
            
        glPushMatrix() # remember previous matrix state before translating, rotating
        glTranslatef(self.x, self.y, -self.z) # translate to point to draw

        glBegin(GL_POINTS) # draw point
        glVertex3f(0.0, 0.0, 0.0)
        glEnd()

        glPopMatrix() # back to previous matrix state

    def intersects(self, x,y):
        if x==self.x and y==self.y : return True

class Circle(Base):
    """ Circle class
        Circle(x=20, y=100, z=1, width=300, color=(1,1,0,0.3), stroke=5, rotation=0, style=GLU_FILL)
        style choices are : GLU_LINE, GLU_FILL, GLU_SILHOUETTE, GLU_POINT
    """
    def __init__(self, x=10, y=10, z=0, width=2, color=(0,0,0,1), stroke=0, rotation=0.0, style=GLU_FILL):
        self.radius = width*0.5
        self.rect = Rect(x, y, width, width)
        self.style = style
        self.circleresolution = 60
        Base.__init__(self, x,y,z,color, stroke, rotation)
        
    def setWidth(self, w):
        self.radius = w*0.5
        self.rect.width = w
    width = property(Base.getWidth, setWidth)
        
    def render(self):
        """ Draw Circle
            x, y, z, width in pixel, rotation, color and line width in px
            style choices are : GLU_LINE, GLU_FILL, GLU_SILHOUETTE, GLU_POINT
            TO DO : textured circles
        """
        
        glColor4f(*self.color)
        glPushMatrix()

        glTranslatef(self.x, self.y, -self.z)
        glRotatef(self.rotation, 0, 0, 0.1)


        if self.radius < 1 : self.radius = 1

        if self.stroke :
            inner = self.radius - self.stroke # outline width
            if inner < 0: inner=0
        else :
             inner = 0 # filled
        
        gluQuadricDrawStyle(self.q, self.style)

        gluDisk(self.q, inner, self.radius, self.circleresolution, 1) # gluDisk(quad, inner, outer, slices, loops)
            
        glPopMatrix()



class Arc(Base):
    """ Arc class
        Arc(x=10, y=10, z=0, radius=1, start=0, sweep=1, color=(0,0,0,1), stroke=0, rotation=0.0, style=GLU_FILL)
        style choices are : GLU_LINE, GLU_FILL, GLU_SILHOUETTE, GLU_POINT
    """
    def __init__(self, x=10, y=10, z=0, radius=1, start=0, sweep=1, color=(0,0,0,1), stroke=0,
                 rotation=0.0, style=GLU_FILL):

        Base.__init__(self, x,y,z,color, stroke, rotation)
        self.radius = radius
        self.start = start
        self.sweep = sweep
        self.style = style
        self.circleresolution = 60
        
    def render(self):
        """
        Draws Arc
        """
        glColor4f(*self.color)
        glPushMatrix()

        glTranslatef(self.x, self.y, -self.z)
        glRotatef(self.rotation, 0, 0, 0.1)

        if self.stroke : 
            inner = self.radius - self.stroke
            if inner < 0: inner=0
        else :
            inner = 0 # full, no inner
        #self.start -= 180
        
        gluQuadricDrawStyle(self.q, self.style)
        
        gluPartialDisk(self.q, inner, self.radius, self.circleresolution, 1, self.start, self.sweep)
        
        glPopMatrix()



class Polygon(Base):
    def __init__(self, v, z=0, color=(0,0,0,1), stroke=0, rotation=0.0, style=0):
        """ polygon class
            Polygon(vertexarray=[(0, 0), (29, 100), (30, 200)], z=100, color=(0,0.3,0.1,1), stroke=0, rotation=23)
            overwrites few methods from superclass as polygons are more complex, needs to update everyvertex.
        """
        self.v = v
        l, t, r, b = calcPolygonRect(v) # get the bounding rect
        self.rect = Rect(l+(r-l)*0.5, t+(b-t)*0.5, r-l, b-t)
        self.v2 = [(i[0] - self.rect.x, i[1] - self.rect.y) for i in v] #relative polygon

        self.style = style
        
        Base.__init__(self, self.rect.x, self.rect.y, z,color,stroke,rotation)

    def updateV(self):
        self.v = [(self.rect.x + n[0], self.rect.y + n[1]) for n in self.v2]

    def setLoc(self, p):
        self.rect.loc = p ; self.updateV()
    def setX(self, x):
        self.rect.x = x ; self.updateV()
    def setY(self, y):
        self.rect.y = y; self.updateV()
    x = property(Base.getX, setX)
    y = property(Base.getY, setY)
    loc = property(Base.getLoc, setLoc)
    
    def render(self):
        """ Draw Polygon
            v is an array with tuple points like [(x, y), (x2, y2), (x3, y3)]
            min vertex number to draw a polygon is 3
            stroke=0 to fil with color the inside of the shape or stroke=N just to draw N-px thick outline.
        """
        l,t,r,b = calcPolygonRect(self.v)
        x,y = calcRectCenter(l,t,r,b)
        self.drawVertex(x, y, self.z, [(i[0] - x, i[1] - y) for i in self.v], self.color, self.stroke, self.rotation, self.style)

    
    def drawVertex(self, x, y,  z=0, v=(), color=(0,0,0,1), stroke=0, rotation=0.0,   style=0):
        
        glColor4f(*self.color)    
        glPushMatrix()

        glTranslatef(x, y, -z)
        glRotatef(self.rotation, 0, 0, 0.1)

        if self.style :
            glEnable(GL_LINE_STIPPLE)
            glLineStipple(1, style)

        # -- start drawing
        if self.stroke : # outlined polygon
            glLineWidth(self.stroke)
            glBegin(GL_LINE_LOOP)
        else: # filled polygon
            if   len(v) == 4 : glBegin(GL_QUADS)
            elif len(v)  > 4 : glBegin(GL_POLYGON)
            else :             glBegin(GL_TRIANGLES) # which type of polygon are we drawing?

        for p in v:
            glVertex3f(p[0], p[1],0)  # draw each vertex

        glEnd()
        # -- end drawing
        
        if self.style : glDisable(GL_LINE_STIPPLE)
        
        glPopMatrix()


class LineRel(Base):
    def __init__(self, x,y, a=(0,0), b=(0,0), z=0, color=(0,0,0,1), stroke=0, rotation=0.0, style=0):
        """ Draws a basic line given the begining and end point (tuples), color (tuple) and stroke
            (thickness of line)
            Line( x,y, a=(1,1), b=(100,100), z=0, color=(0.2,0,0,1), stroke=10, rotation=45)
        """
        w = (b[0] - a[0]) 
        h = (b[1] - a[1]) 
        x = abs(a[0] + w*0.5)
        y = abs(a[1] + h*0.5)
        self.a2 = abs(a[0]) - x, abs(a[1]) - y
        self.b2 = abs(b[0]) - x, abs(b[1]) - y
        self.a = x - w*0.5, y - w*0.5
        self.b = x + w*0.5, y + w*0.5
        self.rect = Rect(x, y, w, h)
        self.style = style
        Base.__init__(self, x, y, z,color,stroke,rotation)

    def render(self):
        """
        Draws Line
        """
        p1 = self.a2
        p2 = self.b2
        glColor4f(*self.color)
        color  = (GLfloat *4)(*self.color)
               
        glPushMatrix()

        glTranslatef(self.x, self.y, -self.z) # translate to GL loc point
        glRotatef(self.rotation, 0, 0, 0.1)

        if self.style :
            glEnable(GL_LINE_STIPPLE)
            glLineStipple(1, self.style)
##        else :
##            glDisable(GL_LINE_STIPPLE)
            
        if self.stroke <= 0:
            self.stroke = 1
        glLineWidth(self.stroke)

        glBegin(GL_LINES)
        glVertex2i(int(p1[0]), int(p1[1])) # draw pixel points
        glVertex2i(int(p2[0]), int(p2[1]))
        glEnd()

        if self.style :
            glDisable(GL_LINE_STIPPLE)
        
        glPopMatrix()


    def updateAB(self):
        self.a = self.x + self.a[0], self.y + self.a[0]
        self.b = self.x + self.b[0], self.y + self.b[0]

    def setLoc(self, p):
        self.rect.loc = p ; self.updateAB()
    def setX(self, x):
        self.rect.x = x ; self.updateAB()
    def setY(self, y):
        self.rect.y = y; self.updateAB()
    x = property(Base.getX, setX)
    y = property(Base.getY, setY)
    loc = property(Base.getLoc, setLoc)


class Line(LineRel):
    def __init__(self, a=(0,0), b=(0,0), z=0, color=(0,0,0,1), stroke=0, rotation=0.0, style=0):
        """ Draws a basic line given the begining and end point (tuples), color (tuple) and stroke
            (thickness of line)
            Line( a=(1,1), b=(100,100), z=20, color=(0.2,0,0,1), stroke=10, rotation=45)
        """
        w = (b[0] - a[0]) 
        h = (b[1] - a[1]) 
        x = abs(a[0] + w*0.5) # abs x,y
        y = abs(a[1] + h*0.5)
        a = x-w*0.5, y-h*0.5 # relative a,b
        b = x+w*0.5, y+h*0.5
        LineRel.__init__(self, x, y, a, b, z, color, stroke, rotation, style)

        

class Curve(Base):
    
    def __init__(self, data, stride=0.1,Type=GLU_MAP1_TRIM_2):
        self.nurbObject = gluNewNurbsRenderer()
        gluNurbsProperty( self.nurbObject, GLU_SAMPLING_TOLERANCE, 25.0 )
        gluNurbsProperty( self.nurbObject, GLU_DISPLAY_MODE, GLU_FILL )
        #gl2df = lambda data: ((GLfloat * len(data[0])) * len(data))(*data)
        self.data = self.tconv(data)
        self.cnt = len(data)
        self.stride = 4
        self.Type = Type
    
    def tconv(self, data):
        """
        data: List of tuples
        """
        ar = ((GLfloat*len(data[0]))*len(data))(*data)
        cdata = ctypes.cast(ar, ctypes.POINTER(GLfloat))
        return cdata
        
    def render(self):
        knotVector = (GLfloat * 8)(*(0, 0, 0, 0, 1, 1, 1, 1))
        gluBeginCurve(self.nurbObject)
        gluNurbsCurve(self.nurbObject,8, knotVector, 3,self.data,4,GL_MAP1_VERTEX_3)
        gluEndCurve(self.nurbObject)
    

class Rect(object):
    def __init__(self, x=0,y=0,w=0,h=0):
        """ rect(self, x=0,y=0,w=0,h=0)
            x,y,loc, width, height
            left,top,right,bottom
            quad ->
            --------------------
            topleft = quad[0]
            topright = quad[1]
            bottomright = quad[2]
            bottomleft = quad[3]
        """
        self.rect = x,y,w,h

    def setRect(self, r):
        self.__x = r[0]
        self.__y = r[1]
        self.__width = r[2]
        self.__height = r[3]
        w = r[2]*0.5 ; h = r[3]*0.5
        self.__rect = r[0]-w, r[1]-h, r[0]+w, r[1]+h # l t r b
    def getRect(self):
        return self.__rect
    rect = property(getRect, setRect)

    def setQuad(self, q): # [ q[0][0], q[0][1], q[1][0], q[2][1] ] # l t r b
        self.rect = q[0][0]+(q[1][0]-q[0][0])*0.5, q[0][1]+(q[2][1]-q[0][1])*0.5, q[1][0]-q[0][0], q[2][1]-q[0][1] 
    def getQuad(self):
        return [(self.rect[0], self.rect[1]),(self.rect[2], self.rect[1]),(self.rect[2], self.rect[3]),(self.rect[0], self.rect[3])] # tl tr br bl
    quad = property(getQuad, setQuad)

    def setX(self, x):
        self.rect = x, self.y, self.width, self.height
    def getX(self) : return self.__x
    x = property(getX, setX)

    def setY(self, y):
        self.rect = self.x, y, self.width, self.height
    def getY(self) : return self.__y
    y = property(getY, setY)

    def setLoc(self, p):
        self.rect = p[0], p[1], self.width, self.height
    def getLoc(self) : return self.__x, self.__y # self.x, self.y
    loc = property(getLoc, setLoc)

    def setWidth(self, w):
        self.rect = self.x, self.y, w, self.height
    def getWidth(self): return self.__width
    width = property(getWidth, setWidth)
    
    def setHeight(self, h):
        self.rect = self.x, self.y, self.width, h
    def getHeight(self): return self.__height
    height = property(getHeight, setHeight)

    def setLeft(self,x):
        self.rect = x+self.width*0.5, self.y, self.width, self.height
    def getLeft(self): return self.rect[0]
    left = property(getLeft, setLeft)
    
    def setTop(self,y):
        self.rect = self.x, y+self.width*0.5, self.width, self.height
    def getTop(self): return self.rect[1]
    top = property(getTop, setTop)
    
    def setRight(self,x):
        self.rect = x-self.width*0.5, self.y, self.width, self.height
    def getRight(self): return self.rect[2]
    right = property(getRight, setRight)
    
    def setBottom(self,x):
        self.rect = self.x, y-self.width*0.5, self.width, self.height
    def getBottom(self): return self.rect[3]
    bottom = property(getBottom, setBottom)


def calcPolygonRect(pointArray):
    """ receives a point list and returns the rect that contains them as a tupple -> tuple left, top, right, bottom
    """
    # init to ridiculously big values. not very elegant or eficient
    l, t, r, b = 10000000, 10000000, -10000000, -10000000
##    l = pointArray[0]
##    t = pointArray[1]
##    r = l
##    b = t

    for n in pointArray: # calc bounding rectangle rect
        if n[0] < l : l = n[0]
        if n[0] > r : r = n[0]
        if n[1] < t : t = n[1]
        if n[1] > b : b = n[1]

    return l, t, r, b


def calcRectCenter(l,t,r,b):#,v=()):
    """ returns rect center point -> x,y
        calcRectCenter(l,t,r,b)
    """
##    if len(v) : l,t,r,b = v[0],v[1],v[2],v[3]
    return l+((r-l)*0.5), t+((b-t)*0.5)

if __name__=="__main__":
    import random
    win = window.Window()
    glEnable(GL_BLEND)
    
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    p = Pixel(110,10)
    c = Circle(300,100,width=100,color=(0.,.9,0.,1.))
    a = Arc(250,150,radius=100,color=(1.,0.,0.,1.),sweep=90,style=GLU_FILL)
    P = Polygon([(100, 0), (150, 200), (180, 200),(160,100),(200,5)],color=(.3,0.2,0.5,.7))
    l = Line((110,299),(200,299),stroke=2,color=(0,0.,1.,1.))
    g= Group([c, a, P, l])
    C = Curve(data=[(10,50),(20,60),(30,40),(40,60)])
    while not win.has_exit:
        win.dispatch_events()
        mvmt = random.randint(-10, 10)
        angle = .2
        col = [random.random() for i in xrange(3)]+[1]
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        c.color = col
        c.render()
        p.render()
        a.render()
        a.rotation+=1
        P.render()
        l.render()
        C.render()
        #g.move(mvmt)
        g.rotate(angle)
        win.flip()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/flavio/Documents/Projetos/Epigrass/Epigrass-devel/cpanel.ui'
#
# Created: Qui Jun 28 18:44:11 2007
#      by: The PyQt User Interface Compiler (pyuic) 3.17
#
# WARNING! All changes made in this file will be lost!


import sys
from qt import *

image0_data = \
    "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
    "\x49\x48\x44\x52\x00\x00\x00\x20\x00\x00\x00\x20" \
    "\x08\x06\x00\x00\x00\x73\x7a\x7a\xf4\x00\x00\x00" \
    "\xdc\x49\x44\x41\x54\x58\x85\xed\x96\x59\x0e\xc3" \
    "\x20\x0c\x44\x1f\x55\xef\x55\x1f\x9d\x9b\x39\x1f" \
    "\x55\x22\x67\x81\x62\x16\xd1\x56\x19\x29\x5f\x04" \
    "\x66\x0c\xc3\x98\x00\x40\x44\x59\x21\x04\x3c\x68" \
    "\x99\x0b\x3c\xcf\xeb\x45\xbd\xfa\x31\x05\x41\xbc" \
    "\x9c\x79\x01\x82\x78\xab\x70\x09\x4e\xc2\x5b\x79" \
    "\x2f\x3c\x66\x90\xde\x02\x6e\x01\x5f\x25\xe0\x7d" \
    "\xe7\x4b\xd2\xac\x31\xf1\x52\x28\x4e\xc2\xd6\xc4" \
    "\x2b\x16\x90\x49\xc2\xb1\x41\x35\x2b\x09\x4f\x3b" \
    "\x60\x91\x39\x8e\x71\x1e\x38\x92\x5f\x91\xd9\x31" \
    "\x35\x47\x13\x68\x30\xa7\xad\xb6\xf4\x38\x22\x51" \
    "\x15\xb6\xaf\x9a\xdc\x92\x7a\xbd\xd0\x55\xc0\x0c" \
    "\x23\xfe\x46\x12\x66\x8d\xd6\xeb\x4d\xa8\x02\x21" \
    "\xa6\x3c\x20\x86\x6f\x3f\x2e\xc8\x36\xb7\x06\xd3" \
    "\x77\xc0\xac\x53\x6f\xc0\x96\xb9\xd3\x4d\xb8\x43" \
    "\x6d\x25\xff\xb3\x03\xe0\xaf\x66\x48\x78\x79\x7a" \
    "\x41\x2b\x57\xf2\xda\xac\x8b\xbf\x4c\x06\xac\x57" \
    "\x30\xd7\x29\xbb\xe2\xd8\xed\x46\xf4\x8b\x8f\x15" \
    "\x74\xeb\xf7\x09\x2c\x49\xf5\x7d\xcb\x80\x93\xf4" \
    "\xd9\x00\x00\x00\x00\x49\x45\x4e\x44\xae\x42\x60" \
    "\x82"

class MainPanel(QWidget):
    def __init__(self,parent = None,name = None,fl = 0):
        QWidget.__init__(self,parent,name,fl)

        self.image0 = QPixmap()
        self.image0.loadFromData(image0_data,"PNG")
        if not name:
            self.setName("MainPanel")

        self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding,0,0,self.sizePolicy().hasHeightForWidth()))
        self.setMinimumSize(QSize(560,480))
        self.setIcon(self.image0)


        LayoutWidget = QWidget(self,"progress_Layout")
        LayoutWidget.setGeometry(QRect(11,415,538,25))
        progress_Layout = QHBoxLayout(LayoutWidget,6,6,"progress_Layout")

        self.textLabel1_6 = QLabel(LayoutWidget,"textLabel1_6")
        self.textLabel1_6.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Fixed,0,0,self.textLabel1_6.sizePolicy().hasHeightForWidth()))
        progress_Layout.addWidget(self.textLabel1_6)

        self.stepLCD = QLCDNumber(LayoutWidget,"stepLCD")
        self.stepLCD.setSizePolicy(QSizePolicy(QSizePolicy.Minimum,QSizePolicy.Fixed,0,0,self.stepLCD.sizePolicy().hasHeightForWidth()))
        progress_Layout.addWidget(self.stepLCD)

        LayoutWidget_2 = QWidget(self,"Run_Layout")
        LayoutWidget_2.setGeometry(QRect(11,442,538,32))
        Run_Layout = QHBoxLayout(LayoutWidget_2,0,6,"Run_Layout")

        self.buttonHelp = QPushButton(LayoutWidget_2,"buttonHelp")
        self.buttonHelp.setAutoDefault(1)
        Run_Layout.addWidget(self.buttonHelp)
        Horizontal_Spacing2 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        Run_Layout.addItem(Horizontal_Spacing2)

        self.buttonRun = QPushButton(LayoutWidget_2,"buttonRun")
        self.buttonRun.setAutoDefault(1)
        self.buttonRun.setDefault(1)
        Run_Layout.addWidget(self.buttonRun)

        self.buttonExit = QPushButton(LayoutWidget_2,"buttonExit")
        self.buttonExit.setAutoDefault(1)
        Run_Layout.addWidget(self.buttonExit)

        self.tabWidget = QTabWidget(self,"tabWidget")
        self.tabWidget.setEnabled(1)
        self.tabWidget.setGeometry(QRect(11,6,538,407))
        self.tabWidget.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.MinimumExpanding,0,0,self.tabWidget.sizePolicy().hasHeightForWidth()))

        self.Widget8 = QWidget(self.tabWidget,"Widget8")

        self.splitter6 = QSplitter(self.Widget8,"splitter6")
        self.splitter6.setGeometry(QRect(10,0,520,360))
        self.splitter6.setOrientation(QSplitter.Vertical)

        self.modSpec = QGroupBox(self.splitter6,"modSpec")
        self.modSpec.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.MinimumExpanding,0,0,self.modSpec.sizePolicy().hasHeightForWidth()))
        self.modSpec.setMinimumSize(QSize(518,80))

        LayoutWidget_3 = QWidget(self.modSpec,"layout7")
        LayoutWidget_3.setGeometry(QRect(10,20,500,60))
        layout7 = QGridLayout(LayoutWidget_3,1,1,6,6,"layout7")

        self.chooseButton = QPushButton(LayoutWidget_3,"chooseButton")

        layout7.addWidget(self.chooseButton,1,2)

        self.textLabel1 = QLabel(LayoutWidget_3,"textLabel1")

        layout7.addWidget(self.textLabel1,0,0)
        spacer4 = QSpacerItem(370,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout7.addMultiCell(spacer4,0,0,1,3)

        self.scriptNameEdit = QLineEdit(LayoutWidget_3,"scriptNameEdit")
        self.scriptNameEdit.setMargin(1)

        layout7.addMultiCellWidget(self.scriptNameEdit,1,1,0,1)

        self.editButton = QPushButton(LayoutWidget_3,"editButton")

        layout7.addWidget(self.editButton,1,3)

        LayoutWidget_4 = QWidget(self.splitter6,"Database_layout")
        Database_layout = QGridLayout(LayoutWidget_4,1,1,6,6,"Database_layout")
        spacer3 = QSpacerItem(20,100,QSizePolicy.Minimum,QSizePolicy.Expanding)
        Database_layout.addItem(spacer3,1,0)

        self.dbSpecGroupBox = QGroupBox(LayoutWidget_4,"dbSpecGroupBox")
        self.dbSpecGroupBox.setEnabled(0)
        self.dbSpecGroupBox.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,QSizePolicy.MinimumExpanding,0,0,self.dbSpecGroupBox.sizePolicy().hasHeightForWidth()))

        LayoutWidget_5 = QWidget(self.dbSpecGroupBox,"layout7")
        LayoutWidget_5.setGeometry(QRect(10,20,360,140))
        layout7_2 = QGridLayout(LayoutWidget_5,1,1,6,6,"layout7_2")

        self.textLabel2 = QLabel(LayoutWidget_5,"textLabel2")
        self.textLabel2.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Fixed,0,0,self.textLabel2.sizePolicy().hasHeightForWidth()))

        layout7_2.addWidget(self.textLabel2,0,0)

        self.portEdit = QLineEdit(LayoutWidget_5,"portEdit")

        layout7_2.addWidget(self.portEdit,1,1)

        self.textLabel3 = QLabel(LayoutWidget_5,"textLabel3")
        self.textLabel3.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Fixed,0,0,self.textLabel3.sizePolicy().hasHeightForWidth()))

        layout7_2.addWidget(self.textLabel3,0,1)

        self.pwEdit = QLineEdit(LayoutWidget_5,"pwEdit")
        self.pwEdit.setEchoMode(QLineEdit.Password)

        layout7_2.addWidget(self.pwEdit,3,1)

        self.textLabel1_2 = QLabel(LayoutWidget_5,"textLabel1_2")
        self.textLabel1_2.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel1_2.sizePolicy().hasHeightForWidth()))

        layout7_2.addWidget(self.textLabel1_2,2,1)

        self.textLabel4 = QLabel(LayoutWidget_5,"textLabel4")
        self.textLabel4.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel4.sizePolicy().hasHeightForWidth()))

        layout7_2.addWidget(self.textLabel4,2,0)

        self.hostnEdit = QLineEdit(LayoutWidget_5,"hostnEdit")

        layout7_2.addWidget(self.hostnEdit,1,0)

        self.uidEdit = QLineEdit(LayoutWidget_5,"uidEdit")

        layout7_2.addWidget(self.uidEdit,3,0)

        Database_layout.addMultiCellWidget(self.dbSpecGroupBox,0,1,1,1)

        layout5 = QVBoxLayout(None,0,6,"layout5")

        self.textLabel1_11 = QLabel(LayoutWidget_4,"textLabel1_11")
        self.textLabel1_11.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred,0,0,self.textLabel1_11.sizePolicy().hasHeightForWidth()))
        layout5.addWidget(self.textLabel1_11)

        self.dbType = QComboBox(0,LayoutWidget_4,"dbType")
        self.dbType.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed,0,0,self.dbType.sizePolicy().hasHeightForWidth()))
        layout5.addWidget(self.dbType)

        Database_layout.addLayout(layout5,0,0)
        self.tabWidget.insertTab(self.Widget8,QString.fromLatin1(""))

        self.Widget9 = QWidget(self.tabWidget,"Widget9")

        LayoutWidget_6 = QWidget(self.Widget9,"layout9")
        LayoutWidget_6.setGeometry(QRect(9,19,294,132))
        layout9 = QGridLayout(LayoutWidget_6,1,1,6,6,"layout9")

        self.unameEdit = QLineEdit(LayoutWidget_6,"unameEdit")

        layout9.addMultiCellWidget(self.unameEdit,0,0,1,3)

        self.textLabel1_5 = QLabel(LayoutWidget_6,"textLabel1_5")

        layout9.addMultiCellWidget(self.textLabel1_5,3,3,0,1)

        self.langCombo = QComboBox(0,LayoutWidget_6,"langCombo")

        layout9.addMultiCellWidget(self.langCombo,3,3,2,3)

        self.textLabel5 = QLabel(LayoutWidget_6,"textLabel5")

        layout9.addWidget(self.textLabel5,1,0)

        self.textLabel2_3 = QLabel(LayoutWidget_6,"textLabel2_3")

        layout9.addMultiCellWidget(self.textLabel2_3,2,2,0,2)

        self.textLabel2_2 = QLabel(LayoutWidget_6,"textLabel2_2")

        layout9.addWidget(self.textLabel2_2,0,0)

        self.editorEdit = QLineEdit(LayoutWidget_6,"editorEdit")

        layout9.addMultiCellWidget(self.editorEdit,1,1,1,3)

        self.pdfEdit = QLineEdit(LayoutWidget_6,"pdfEdit")

        layout9.addWidget(self.pdfEdit,2,3)
        self.tabWidget.insertTab(self.Widget9,QString.fromLatin1(""))

        self.TabPage = QWidget(self.tabWidget,"TabPage")

        self.line1 = QFrame(self.TabPage,"line1")
        self.line1.setGeometry(QRect(289,34,20,320))
        self.line1.setFrameShape(QFrame.VLine)
        self.line1.setFrameShadow(QFrame.Sunken)
        self.line1.setLineWidth(3)
        self.line1.setFrameShape(QFrame.VLine)

        LayoutWidget_7 = QWidget(self.TabPage,"layout14")
        LayoutWidget_7.setGeometry(QRect(0,10,280,350))
        layout14 = QVBoxLayout(LayoutWidget_7,6,6,"layout14")

        self.textLabel5_2 = QLabel(LayoutWidget_7,"textLabel5_2")
        layout14.addWidget(self.textLabel5_2)

        self.textEdit1 = QTextEdit(LayoutWidget_7,"textEdit1")
        layout14.addWidget(self.textEdit1)

        LayoutWidget_8 = QWidget(self.TabPage,"layout18")
        LayoutWidget_8.setGeometry(QRect(320,10,200,349))
        layout18 = QVBoxLayout(LayoutWidget_8,6,6,"layout18")

        layout16 = QVBoxLayout(None,0,6,"layout16")

        layout11 = QVBoxLayout(None,0,6,"layout11")

        self.textLabel1_3 = QLabel(LayoutWidget_8,"textLabel1_3")
        self.textLabel1_3.setAlignment(QLabel.AlignCenter)
        layout11.addWidget(self.textLabel1_3)

        self.dbBackup = QPushButton(LayoutWidget_8,"dbBackup")
        self.dbBackup.setEnabled(0)
        layout11.addWidget(self.dbBackup)

        self.dbInfo = QPushButton(LayoutWidget_8,"dbInfo")
        self.dbInfo.setEnabled(0)
        layout11.addWidget(self.dbInfo)
        layout16.addLayout(layout11)

        self.line2 = QFrame(LayoutWidget_8,"line2")
        self.line2.setFrameShape(QFrame.HLine)
        self.line2.setFrameShadow(QFrame.Sunken)
        self.line2.setLineWidth(2)
        self.line2.setFrameShape(QFrame.HLine)
        layout16.addWidget(self.line2)

        layout15 = QVBoxLayout(None,0,6,"layout15")

        self.textLabel1_4 = QLabel(LayoutWidget_8,"textLabel1_4")
        self.textLabel1_4.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Preferred,0,0,self.textLabel1_4.sizePolicy().hasHeightForWidth()))
        self.textLabel1_4.setMargin(0)
        self.textLabel1_4.setAlignment(QLabel.AlignCenter)
        layout15.addWidget(self.textLabel1_4)

        self.repOpen = QPushButton(LayoutWidget_8,"repOpen")
        layout15.addWidget(self.repOpen)
        layout16.addLayout(layout15)
        layout18.addLayout(layout16)
        spacer6 = QSpacerItem(20,170,QSizePolicy.Minimum,QSizePolicy.Expanding)
        layout18.addItem(spacer6)
        self.tabWidget.insertTab(self.TabPage,QString.fromLatin1(""))

        self.visPage = QWidget(self.tabWidget,"visPage")

        LayoutWidget_9 = QWidget(self.visPage,"layout24")
        LayoutWidget_9.setGeometry(QRect(0,279,530,71))
        layout24 = QHBoxLayout(LayoutWidget_9,6,6,"layout24")

        layout23 = QVBoxLayout(None,0,6,"layout23")

        self.textLabel1_9 = QLabel(LayoutWidget_9,"textLabel1_9")
        self.textLabel1_9.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel1_9.sizePolicy().hasHeightForWidth()))
        layout23.addWidget(self.textLabel1_9)

        self.consensusButton = QPushButton(LayoutWidget_9,"consensusButton")
        self.consensusButton.setEnabled(0)
        layout23.addWidget(self.consensusButton)
        layout24.addLayout(layout23)
        spacer8 = QSpacerItem(260,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout24.addItem(spacer8)

        LayoutWidget_10 = QWidget(self.visPage,"layout20")
        LayoutWidget_10.setGeometry(QRect(0,10,265,263))
        layout20 = QVBoxLayout(LayoutWidget_10,6,6,"layout20")

        self.textLabel1_7 = QLabel(LayoutWidget_10,"textLabel1_7")
        self.textLabel1_7.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel1_7.sizePolicy().hasHeightForWidth()))
        layout20.addWidget(self.textLabel1_7)

        self.tableList = QComboBox(0,LayoutWidget_10,"tableList")
        layout20.addWidget(self.tableList)

        self.textLabel1_10 = QLabel(LayoutWidget_10,"textLabel1_10")
        self.textLabel1_10.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel1_10.sizePolicy().hasHeightForWidth()))
        layout20.addWidget(self.textLabel1_10)

        self.variableList = QComboBox(0,LayoutWidget_10,"variableList")
        layout20.addWidget(self.variableList)

        self.textLabel2_4 = QLabel(LayoutWidget_10,"textLabel2_4")
        self.textLabel2_4.setEnabled(1)
        self.textLabel2_4.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel2_4.sizePolicy().hasHeightForWidth()))
        layout20.addWidget(self.textLabel2_4)

        self.mapList = QComboBox(0,LayoutWidget_10,"mapList")
        self.mapList.setEnabled(1)
        layout20.addWidget(self.mapList)

        LayoutWidget_11 = QWidget(self.visPage,"layout19")
        LayoutWidget_11.setGeometry(QRect(272,11,260,261))
        layout19 = QVBoxLayout(LayoutWidget_11,6,6,"layout19")

        self.dbscanButton = QPushButton(LayoutWidget_11,"dbscanButton")
        layout19.addWidget(self.dbscanButton)

        self.textLabel1_8 = QLabel(LayoutWidget_11,"textLabel1_8")
        self.textLabel1_8.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.textLabel1_8.sizePolicy().hasHeightForWidth()))
        layout19.addWidget(self.textLabel1_8)

        self.rateSpinBox = QSpinBox(LayoutWidget_11,"rateSpinBox")
        self.rateSpinBox.setMinValue(20)
        layout19.addWidget(self.rateSpinBox)
        spacer7 = QSpacerItem(20,90,QSizePolicy.Minimum,QSizePolicy.Expanding)
        layout19.addItem(spacer7)

        self.playButton_2D = QPushButton(LayoutWidget_11,"playButton_2D")
        self.playButton_2D.setEnabled(0)
        layout19.addWidget(self.playButton_2D)

        self.playButton = QPushButton(LayoutWidget_11,"playButton")
        layout19.addWidget(self.playButton)
        self.tabWidget.insertTab(self.visPage,QString.fromLatin1(""))

        self.languageChange()

        self.resize(QSize(560,480).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)


    def languageChange(self):
        self.setCaption(self.__tr("Epigrass Control Panel"))
        self.textLabel1_6.setText(self.__tr("Progress:"))
        QToolTip.add(self.stepLCD,self.__tr("Simulation step"))
        self.buttonHelp.setText(self.__tr("&Help"))
        self.buttonHelp.setAccel(QKeySequence(self.__tr("F1")))
        QToolTip.add(self.buttonHelp,self.__tr("Click here to open the userguide in the web browser"))
        self.buttonRun.setText(self.__tr("&Run"))
        self.buttonRun.setAccel(QKeySequence(self.__tr("Alt+R")))
        QToolTip.add(self.buttonRun,self.__tr("Click here to start your simulation"))
        self.buttonExit.setText(self.__tr("&Exit"))
        self.buttonExit.setAccel(QKeySequence(self.__tr("Alt+E")))
        QToolTip.add(self.buttonExit,self.__tr("Click here to leave Epigrass"))
        QToolTip.add(self.tabWidget,QString.null)
        self.modSpec.setTitle(self.__tr("Model Specification"))
        self.chooseButton.setText(self.__tr("Choose"))
        self.textLabel1.setText(self.__tr("Script Name:"))
        QToolTip.add(self.scriptNameEdit,self.__tr("write the name of the your script or press the choose button on the right to select one."))
        self.editButton.setText(self.__tr("Edit"))
        self.dbSpecGroupBox.setTitle(self.__tr("Database Specification"))
        self.textLabel2.setText(self.__tr("Host:"))
        self.portEdit.setText(self.__tr("3306"))
        QToolTip.add(self.portEdit,self.__tr("Enter the port  the server listens to."))
        self.textLabel3.setText(self.__tr("Port:"))
        QToolTip.add(self.pwEdit,self.__tr("Database password for the userid entered"))
        self.textLabel1_2.setText(self.__tr("Password:"))
        self.textLabel4.setText(self.__tr("Userid:"))
        self.hostnEdit.setText(self.__tr("localhost"))
        QToolTip.add(self.hostnEdit,self.__tr("This is the url of your database server."))
        QToolTip.add(self.uidEdit,self.__tr("This is the userid for accessing the database server"))
        self.textLabel1_11.setText(self.__tr("Database type:"))
        self.dbType.clear()
        self.dbType.insertItem(self.__tr("MySQL"))
        self.dbType.insertItem(self.__tr("SQLite"))
        self.dbType.insertItem(self.__tr("CSV"))
        self.dbType.setCurrentItem(1)
        QToolTip.add(self.dbType,self.__tr("Select your database type"))
        self.tabWidget.changeTab(self.Widget8,self.__tr("Run Options"))
        QToolTip.add(self.unameEdit,self.__tr("Enter your full name. This will be added to the report."))
        self.textLabel1_5.setText(self.__tr("Language"))
        self.langCombo.clear()
        self.langCombo.insertItem(self.__tr("English"))
        self.langCombo.insertItem(self.__tr("Brazilian portuguese"))
        self.langCombo.insertItem(self.__tr("French"))
        QToolTip.add(self.langCombo,self.__tr("Select the language for the GUI"))
        self.textLabel5.setText(self.__tr("Editor"))
        self.textLabel2_3.setText(self.__tr("PDF Viewer"))
        self.textLabel2_2.setText(self.__tr("Name"))
        QToolTip.add(self.editorEdit,self.__tr("Enter your preferred text editor"))
        self.pdfEdit.setText(QString.null)
        QToolTip.add(self.pdfEdit,self.__tr("Enter the name of your preferred PDF viewer"))
        self.tabWidget.changeTab(self.Widget9,self.__tr("Settings"))
        self.textLabel5_2.setText(self.__tr("Simulation Status"))
        self.textLabel1_3.setText(self.__tr("Database"))
        self.dbBackup.setText(self.__tr("Backup"))
        QToolTip.add(self.dbBackup,self.__tr("Click here to backup the epigrass database "))
        self.dbInfo.setText(self.__tr("Info"))
        QToolTip.add(self.dbInfo,self.__tr("Click here for a short description of the epigrass database"))
        self.textLabel1_4.setText(self.__tr("Report"))
        self.repOpen.setText(self.__tr("Open"))
        self.tabWidget.changeTab(self.TabPage,self.__tr("Utilities"))
        self.textLabel1_9.setText(self.__tr("Spread Trees"))
        self.consensusButton.setText(self.__tr("Consensus Tree"))
        QToolTip.add(self.consensusButton,self.__tr("Select a directory with tree-files to build consensus on."))
        self.textLabel1_7.setText(self.__tr("Simulations stored:"))
        QToolTip.add(self.tableList,self.__tr("Select a database stored simulation"))
        self.textLabel1_10.setText(self.__tr("Variable to display:"))
        QToolTip.add(self.variableList,self.__tr("Select a variable to display in the animation"))
        self.textLabel2_4.setText(self.__tr("Maps available:"))
        self.mapList.clear()
        self.mapList.insertItem(self.__tr("No map"))
        QToolTip.add(self.mapList,self.__tr("Select a map"))
        self.dbscanButton.setText(self.__tr("Scan DB"))
        self.textLabel1_8.setText(self.__tr("Animation rate"))
        self.playButton_2D.setText(self.__tr("Start 2D animation"))
        self.playButton.setText(self.__tr("Start 3D animation"))
        self.tabWidget.changeTab(self.visPage,self.__tr("Visualization"))


    def Main_layout_destroyed(self,a0):
        print "MainPanel.Main_layout_destroyed(QObject*): Not implemented yet"

    def __tr(self,s,c = None):
        return qApp.translate("MainPanel",s,c)

if __name__ == "__main__":
    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))
    w = MainPanel()
    a.setMainWidget(w)
    w.show()
    a.exec_loop()



# -*- coding: utf-8 -*-

"""
Module implementing MainWindow.
"""
from PyQt4 import QtCore, QtGui
from PyQt4.QtGui import QMainWindow
from PyQt4.QtCore import pyqtSignature

from Ui_neteditor import Ui_MainWindow
from data_io import  loadData
import networkx as nx
#import dgraph
import math
from numpy import array
import elasticnodes as dgraph

class MainWindow(QMainWindow, Ui_MainWindow):
    """
    Class documentation goes here.
    """
    def __init__(self, parent = None):
        """
        Constructor
        """
        QMainWindow.__init__(self, parent)
        self.setupUi(self)
        self.network = None
        self.Map = None
        self.filename = None
    
    @pyqtSignature("QPoint")
    def on_centralWidget_customContextMenuRequested(self, pos):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("QPoint")
    def on_graphicsView_customContextMenuRequested(self, pos):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("QPoint")
    def on_edgeTable_customContextMenuRequested(self, pos):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        pass
    
    @pyqtSignature("")
    def on_edgeTable_itemSelectionChanged(self):
        """
        Slot documentation goes here.
        """
        pass
    
    @pyqtSignature("int, int")
    def on_edgeTable_cellDoubleClicked(self, row, column):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        pass
    
    @pyqtSignature("int, int")
    def on_edgeTable_cellActivated(self, row, column):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        pass
    
    @pyqtSignature("QPoint")
    def on_nodeTable_customContextMenuRequested(self, pos):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("QTableWidgetItem*")
    def on_nodeTable_itemActivated(self, item):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("")
    def on_nodeTable_itemSelectionChanged(self):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("int, int")
    def on_nodeTable_cellActivated(self, row, column):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("QAction*")
    def on_toolBar_actionTriggered(self, action):
        """
        Slot documentation goes here.
        """
        pass
    
    @pyqtSignature("")
    def on_actionOpen_edges_file_activated(self):
        """
        Slot documentation goes here.
        """
        fname = str(QtGui.QFileDialog.getOpenFileName(\
            None,
            self.trUtf8("Select an Epigrass edges file"),
            self.trUtf8("."),
            self.trUtf8("*.csv;;*.*"),
            None))
        self.filename = fname
        self.network = Network(self.graphicsView)
        edgelist = loadData(fname, ',')
        nodelist  = set([])
        header = edgelist.pop(0)
        
        self.edgeTable.setColumnCount(len(header))
        self.edgeTable.setRowCount(len(edgelist))
        self.edgeTable.setHorizontalHeaderLabels(header)
        print self.edgeTable.rowCount()
        for i,e in enumerate(edgelist):
            nodelist.add(e[5])
            nodelist.add(e[6])
            self.network.G.add_edge(e[5], e[6], e[2]) #edge weight is flow S->D
            for j,v in enumerate(e):
                item  = QtGui.QTableWidgetItem(v)
                self.edgeTable.setItem(i,j,item)
        print self.edgeTable.takeItem(0,0).text()
        # Fill nodes table
        self.nodeTable.setColumnCount(1)
        self.nodeTable.setRowCount(len(nodelist))
        self.nodeTable.setHorizontalHeaderLabels(['Geocode'])
        for i,n in enumerate(nodelist):
            item = QtGui.QTableWidgetItem(n)
            self.nodeTable.setItem(i,0,item)
                
    
    @pyqtSignature("")
    def on_actionOpen_Shapefile_activated(self):
        """
        Slot documentation goes here.
        """
        fname = str(QtGui.QFileDialog.getOpenFileName(\
            None,
            self.trUtf8("Select a Shapefile"),
            self.trUtf8("."),
            self.trUtf8("*.shp"),
            None))
    
    @pyqtSignature("")
    def on_action_Save_activated(self):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("")
    def on_actionSave_as_activated(self):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("")
    def on_action_Graph_activated(self):
        """
        Slot documentation goes here.
        """
        if  self.network:
            nlist = []
            for k, v in self.network.getLayout().iteritems():
                nlist.append((k, v[0]*100, v[1]*100, str(k)))
            self.network.drawGraph(nlist, self.network.G.edges())
            
    
    @pyqtSignature("")
    def on_actionE_xit_activated(self):
        """
        Closes the network editor main window
        """
        self.close()
    
    @pyqtSignature("")
    def on_actionStats_activated(self):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("")
    def on_actionCheck_Graph_activated(self):
        """
        Slot documentation goes here.
        """
        # TODO: not implemented yet
        raise NotImplementedError
    
    @pyqtSignature("")
    def on_actionDot_activated(self):
        """
        Slot documentation goes here.
        """
        if self.network:
            nx.write_dot(self.network.G, self.filename.split('.')[0]+'.dot')
    
    @pyqtSignature("")
    def on_actionGML_activated(self):
        """
        Slot documentation goes here.
        """
        if self.network:
            nx.write_gml(self.network.G, self.filename.split('.')[0]+'.gml')
    
    @pyqtSignature("")
    def on_actionPajek_activated(self):
        """
        Slot documentation goes here.
        """
        if self.network:
            nx.write_pajek(self.network.G, self.filename.split('.')[0]+'.pajek')
    
    @pyqtSignature("")
    def on_actionYAML_activated(self):
        """
        Slot documentation goes here.
        """
        if self.network:
            nx.write_yaml(self.network.G, self.filename.split('.')[0]+'.yaml')

class Network:
    '''
    This class handle every network related
    methods, from analysis  to drawing.
    '''
    def __init__(self, displaywidget):
        self.View = displaywidget
        self.View.scene = QtGui.QGraphicsScene(self.View)
        self.N = Graph()
        self.G = nx.XDiGraph(multiedges=True)

    def getLayout(self):
        return nx.random_layout(self.G)
        
    def drawGraph(self, nlist, elist):
        '''
        Draws Graph object representing the network
        '''
        npos= [(n[1], -n[2]) for n in nlist]
        xmin,ymin = array(npos).min(axis=0)
        xmax,ymax = array(npos).max(axis=0)
        for n in nlist:
            node = Node(self.N, n[0], n[3])
            node.setPos(*(n[1], -n[2]))
            node.size = max(xmax-xmin, ymax-ymin)/math.sqrt(len(nlist))*0.5
            self.View.scene.addItem(node)
            self.N.insertNode(node)
            #print node.x(), node.y(), n.center[0], n.center[1]
        self.View.nodes = self.N.nodes
       
        asz = max(xmax-xmin, ymax-ymin)/math.sqrt(len(nlist))*0.2 #arrow size
        for e in elist:
            ed = Edge(self.N.polyDict[e[0]], self.N.polyDict[e[1]])
            ed.arrowSize = asz
            self.View.scene.addItem(ed)
            self.N.insertEdge(ed)
        self.xmax, self.xmin = xmax, xmin
        self.ymax, self.ymin = ymax, ymin
        self.centerScene()
        
    def centerScene(self):
        """
        centers the scene and fits the specified rectangle to it
        """
        ymax, ymin = self.ymax, self.ymin
        xmax, xmin = self.xmax, self.xmin
        xxs = (xmax-xmin)*1.1 #percentage of extra space
        yxs = (ymax-ymin)*1.1 #percentage of extra space
        #calculating center of scene

        xc = (xmax+xmin)/2. 
        yc = (ymax+ymin)/2.
        self.View.scene.setItemIndexMethod(QtGui.QGraphicsScene.NoIndex)
        self.View.scene.setSceneRect(xmin, ymin, xxs, yxs)
#        print self.mapView.scene.width(), self.mapView.scene.height()
        self.View.fitInView(xmin, ymin, xxs, yxs)
        self.View.setScene(self.View.scene)
        self.View.updateSceneRect(self.View.scene.sceneRect())
        self.View.centerOn(xc, yc)
        scale_factor = self.View.width()/xxs
        self.View.scale(scale_factor, scale_factor)
        
        self.View.setCacheMode(QtGui.QGraphicsView.CacheBackground)
        self.View.setRenderHint(QtGui.QPainter.Antialiasing)
        self.View.setTransformationAnchor(QtGui.QGraphicsView.AnchorUnderMouse)
        self.View.setResizeAnchor(QtGui.QGraphicsView.AnchorViewCenter)
        
class Graph(object):
    """
    The Graph.self.data(start)[5]
    """
    def __init__(self, **keywords):
        """
        Construct a Graph.
        """
        self.map = None
        self.nodes = []
        self.edges = []
        self.dragObject = None
        self.click = None
        self.distance = None
        self.timelabel = None
        self.rememberFixed = None
        self.rememberColor = None
#        self.display = display
        self.polyDict = {}#dictionary of nodes by geocode
        self.rect = [0,0,0,0]#xmin,ymin,xmax,ymax

    def insertNode(self, node):
        """
        Insert node into the system.
\        """
        #needs to be a list because the node index in this list identify it within the graph.

        #FIXME: verify time overhead of this check
        if not node in self.nodes:
            self.nodes.append(node)
            self.polyDict[node.geocode] = node
            node.graph = self #pass a reference of self to the node.

    def insertMap(self,map_):
        """
        Insert map into the system.
        """
        self.map = map_
        map_.graph = self

    def insertNodeList(self, nodelist):
        """
        Insert all Nodes in nodelist into the system.
        """

        map(self.insertNode, nodelist)

    def insertEdge(self, edge):
        """
        Insert edge into the system.
        """
        if edge not in self.edges:
            self.edges.append(edge)

    def insertEdgeList(self, edgelist):
        """
        Insert all Edges in edgelist into the system.
        """
        map(self.insertEdge, edgelist)

    def getEdgeFromMatrix(self, matrix):
        """
        Extract edges from the adjacency matrix.
        """
        #FIXME: eitehr integrate this into the graph object, or make it a separate function
        siz = matrix.shape[0]
        el = []
        for c in xrange(siz):
            for l in xrange(c+1): #scans only the lower triangle
                if matrix[l,c]:
                    el.append((c,l))
        return el


    def centerView(self):
        pass

    def getRect(self):
        '''
        Returns the bounding rectangle for the graph
        '''
        for n in self.nodes:
            self.rect[0] = n.pos.x() if n.pos.x()<self.rect[0] else self.rect[0]
            self.rect[1] = n.pos.y() if n.pos.y()<self.rect[1] else self.rect[1]
            self.rect[2] = n.pos.x() if n.pos.x()>self.rect[2] else self.rect[2]
            self.rect[3] = n.pos.y() if n.pos.y()>self.rect[3] else self.rect[3]
        return self.rect


class Node(QtGui.QGraphicsItem):
    Type = QtGui.QGraphicsItem.UserType + 1

    def __init__(self, graphWidget, geocode, name):
        QtGui.QGraphicsItem.__init__(self)

        self.graph = graphWidget
        self.edgeList = []
        self.newPos = QtCore.QPointF()
#        self.setFlag(QtGui.QGraphicsItem.ItemIsMovable)
        self.setZValue(1)
        self.geocode = geocode
        self.name = name
        self.size  = 20
        

    def type(self):
        return Node.Type

    def addEdge(self, edge):
        self.edgeList.append(edge)
        edge.adjust()

    def edges(self):
        return self.edgeList


    def boundingRect(self):
        adjust = 2.0
        return QtCore.QRectF(-10 - adjust, -10 - adjust,
                             23 + adjust, 23 + adjust)

    def shape(self):
        path = QtGui.QPainterPath()
        path.addEllipse(-self.size/2., -self.size/2., self.size,self.size)
        return path

    def paint(self, painter, option, widget):
#        painter.setPen(QtCore.Qt.NoPen)
#        painter.setBrush(QtCore.Qt.darkGray)
#        painter.drawEllipse(-self.size/2.*.8, -self.size/2.*.8, self.size,self.size)

        gradient = QtGui.QRadialGradient(-3, -3, 10)
        if option.state & QtGui.QStyle.State_Sunken:
            gradient.setCenter(3, 3)
            gradient.setFocalPoint(3, 3)
            gradient.setColorAt(1, QtGui.QColor(QtCore.Qt.yellow).light(120))
            gradient.setColorAt(0, QtGui.QColor(QtCore.Qt.darkYellow).light(120))
        else:
            gradient.setColorAt(0, QtCore.Qt.yellow)
            gradient.setColorAt(1, QtCore.Qt.darkYellow)

        painter.setBrush(QtGui.QBrush(gradient))
        painter.setPen(QtGui.QPen(QtCore.Qt.black, 0))
        painter.drawEllipse(-self.size/2., -self.size/2., self.size,self.size)

#        return QtGui.QGraphicsItem.itemChange(self, change, value)

    def mousePressEvent(self, event):
        self.update()
        QtGui.QGraphicsItem.mousePressEvent(self, event)

    def mouseReleaseEvent(self, event):
        self.update()
        QtGui.QGraphicsItem.mouseReleaseEvent(self, event)

class Edge(QtGui.QGraphicsItem):
    Pi = math.pi
    TwoPi = 2.0 * Pi

    Type = QtGui.QGraphicsItem.UserType + 2

    def __init__(self, sourceNode, destNode):
        QtGui.QGraphicsItem.__init__(self)
        self.arrowSize = 10.0
        self.sourcePoint = QtCore.QPointF()
        self.destPoint = QtCore.QPointF()
        self.setAcceptedMouseButtons(QtCore.Qt.NoButton)
        self.source = sourceNode
        self.dest = destNode
        self.source.addEdge(self)
        self.dest.addEdge(self)
        self.adjust()

    def type(self):
        return Edge.Type

    def sourceNode(self):
        return self.source

    def setSourceNode(self, node):
        self.source = node
        self.adjust()

    def destNode(self):
        return self.dest

    def setDestNode(self, node):
        self.dest = node
        self.adjust()

    def adjust(self):
        if not self.source or not self.dest:
            return

        line = QtCore.QLineF(self.mapFromItem(self.source, 0, 0), self.mapFromItem(self.dest, 0, 0))
        length = line.length()
        fac = self.source.size/2.5
        if length == 0.0:
            return

        edgeOffset = QtCore.QPointF((line.dx() * fac) / length, (line.dy() * fac) / length)

        self.prepareGeometryChange()
        self.sourcePoint = line.p1() + edgeOffset
        self.destPoint = line.p2() - edgeOffset

    def boundingRect(self):
        if not self.source or not self.dest:
            return QtCore.QRectF()

        penWidth = 1
        extra = (penWidth + self.arrowSize) / 2.0

        return QtCore.QRectF(self.sourcePoint,
                             QtCore.QSizeF(self.destPoint.x() - self.sourcePoint.x(),
                                           self.destPoint.y() - self.sourcePoint.y())).normalized().adjusted(-extra, -extra, extra, extra)

    def paint(self, painter, option, widget):
        if not self.source or not self.dest:
            return

        # Draw the line itself.
        line = QtCore.QLineF(self.sourcePoint, self.destPoint)

        if line.length() == 0.0:
            return

        painter.setPen(QtGui.QPen(QtCore.Qt.black, .1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        painter.drawLine(line)

        # Draw the arrows if there's enough room.
        angle = math.acos(line.dx() / line.length())
        if line.dy() >= 0:
            angle = Edge.TwoPi - angle

        sourceArrowP1 = self.sourcePoint + QtCore.QPointF(math.sin(angle + Edge.Pi / 3) * self.arrowSize,
                                                          math.cos(angle + Edge.Pi / 3) * self.arrowSize)
        sourceArrowP2 = self.sourcePoint + QtCore.QPointF(math.sin(angle + Edge.Pi - Edge.Pi / 3) * self.arrowSize,
                                                          math.cos(angle + Edge.Pi - Edge.Pi / 3) * self.arrowSize);   
        destArrowP1 = self.destPoint + QtCore.QPointF(math.sin(angle - Edge.Pi / 3) * self.arrowSize,
                                                      math.cos(angle - Edge.Pi / 3) * self.arrowSize)
        destArrowP2 = self.destPoint + QtCore.QPointF(math.sin(angle - Edge.Pi + Edge.Pi / 3) * self.arrowSize,
                                                      math.cos(angle - Edge.Pi + Edge.Pi / 3) * self.arrowSize)

        painter.setBrush(QtCore.Qt.black)
        painter.drawPolygon(QtGui.QPolygonF([line.p1(), sourceArrowP1, sourceArrowP2]))
        painter.drawPolygon(QtGui.QPolygonF([line.p2(), destArrowP1, destArrowP2]))

if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    MainW= QtGui.QMainWindow()
    ui = MainWindow()
    ui.show()
    sys.exit(app.exec_())



# This is a custom model to used in place of Epigrass' built-in models. Custom
# models must always be on a file named CustomModel.py and contain at least 
# a function named Model. Both the File name and the function Names are case-sensitive,
# so be careful. Please refer to the manual for intructions on how to write your 
# own custom models.

def Model(self,vars,par,theta=0, npass=0):
        """
        Calculates the model SIR, and return its values.
        - inits = (E,I,S)
        - par = (Beta, alpha, E,r,delta,B, w, p) see docs.
        - theta = infectious individuals from neighbor sites
        """
        #Defining variable names 
        self.parentSite.vnames = ('Exposed','Infeccious','Susceptible')
        #Initializing
        if self.parentSite.parentGraph.simstep == 1: #get initial values
            E,I,S = (self.bi['e'],self.bi['i'],self.bi['s'])
        else:
            E,I,S = vars
        N = self.parentSite.totpop
        beta,alpha,e,r,delta,B,w,p = par
        #Vacination event (optional)
        if self.parentSite.vaccineNow:
            S -= self.parentSite.vaccov*S
        
        Lpos = beta*S*((I+theta)/(N+npass))**alpha #Number of new cases
        self.parentSite.totalcases += Lpos #update number of cases
        # Model
        Ipos = (1-r)*I + Lpos
        Spos = S + B - Lpos
        Rpos = N-(Spos+Ipos)
        # Updating stats
        self.parentSite.incidence.append(Lpos)
        # Raises site infected flag and adds parent site to the epidemic history list.
        if not self.parentSite.infected: 
            if Lpos > 0:
                #if not self.parentSite.infected:
                self.parentSite.infected = self.parentSite.parentGraph.simstep
                self.parentSite.parentGraph.epipath.append((self.parentSite.parentGraph.simstep,self.parentSite,self.parentSite.infector))
        #Migrating infecctious
        self.parentSite.migInf.append(Ipos)
        
        return [0,Ipos,Spos]



"""
Unit testing script
"""

import unittest
from manager import *
class testObjInstantiation(unittest.TestCase):
    def setUp(self):
        self.sitios = loadData('/home/fccoelho/Documents/Projects_software/epigrass/epigrass/simobj/sitios3.csv',sep=',')
        self.ed=loadData('/home/fccoelho/Documents/Projects_software/epigrass/epigrass/simobj/edgesout.csv',sep=',')
        self.S=simulate()
    def testSites(self):
        l = self.S.instSites(self.sitios)
        for i in range(len(l)):
            self.assertEqual(l[i].sitename,self.sitios[i][2])
            
    def testEdges(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l,self.ed)
        j = 0
        for i in e:
            self.assertEqual((i.source.sitename, i.dest.sitename),(self.ed[j][0],self.ed[j][1]))
            j+=1
    def testGraph(self):
        l = self.S.instSites(self.sitios)
        e = self.S.instEdges(l,self.ed)
        g = self.S.instGraph('grafo',1,l,e)
        
        print g, g.getSiteNames()
if __name__ == '__main__':
    unittest.main()
